@@ -44,22 +44,28 @@ proc Buttons_Group { frame name buts } {
     }
 }
 
-proc Buttons_GroupMenu { menub name labels } {
+proc Buttons_GroupMenu { menu name labels } {
     global buttons
     foreach l $labels {
-	lappend buttons(groupMenu,$name) [list $menub $l]
+	lappend buttons(groupMenu,$name) [list $menu $l]
     }
 }
 
 proc ButtonsGroupState { group state } {
     global buttons
     foreach button $buttons(group,$group) {
-	$button configure -state $state
+	if [catch {$button configure -state $state} err] {
+	    Exmh_Status $err error
+	}
     }
     foreach item $buttons(groupMenu,$group) {
-	set menub [lindex $item 0]
+	set menu [lindex $item 0]
 	set label [lindex $item 1]
-	$menub.m entryconfigure $label -state $state
+	if [catch {$menu entryconfigure $label -state $state} err] {
+	    # The group stuff is not menu-specific, so adding another
+	    # menu results in error messages from this point.
+	    # Exmh_Status $err error
+	}
     }
 }
 
@@ -72,6 +78,7 @@ proc Buttons_Current { curMsg } {
     set buttons(curMsg) $curMsg
     if {$curMsg} {
 	ButtonsGroupState current normal
+	Buttons_Range $curMsg
 	if $buttons(draftMode) {
 	    ButtonsGroupState nodraft disabled
 	    ButtonsGroupState comp normal
@@ -98,11 +105,15 @@ proc Buttons_DraftMode { inDraftMode } {
 	ButtonsGroupState comp normal
 	# Override the Send button
 	if [info exists buttons(comp)] {
-	    set buttons(comp,label) [lindex [$buttons(comp) configure -text] 4]
-	    set buttons(comp,cmd) [lindex [$buttons(comp) configure -command] 4]
-	    $buttons(comp) configure -text EDIT -command Edit_Draft
-	    if {! $buttons(curMsg)} {
-		$buttons(comp) configure -state disabled
+	    if [catch {
+		set buttons(comp,label) [lindex [$buttons(comp) configure -text] 4]
+		set buttons(comp,cmd) [lindex [$buttons(comp) configure -command] 4]
+		$buttons(comp) configure -text EDIT -command Edit_Draft
+		if {! $buttons(curMsg)} {
+		    $buttons(comp) configure -state disabled
+		}
+	    } err] {
+		Exmh_Status $err error
 	    }
 	}
     } else {
@@ -121,8 +132,12 @@ proc Buttons_DraftMode { inDraftMode } {
     return
 }
 
-proc Buttons_Range {} {
-    ButtonsGroupState range normal
+proc Buttons_Range { {ok 1} } {
+    if {$ok} {
+	ButtonsGroupState range normal
+    } else {
+	ButtonsGroupState range disabled
+    }
 }
 
 #####################################################################
@@ -133,10 +148,13 @@ proc Buttons_Main { frame } {
     global buttons
     set buttons(mainF) $frame
 
-    foreach b [concat [option get $frame buttonlist {}] \
-		      [option get $frame ubuttonlist {}]] {
+    foreach b [Widget_GetButDef $frame] {
 	Widget_AddButDef $frame $b
     }
+    foreach M [Widget_GetMenuBDef $frame] {
+	set menu [Widget_AddMenuBDef $frame $M {right padx 1}]
+	ButtonMenuInner $menu
+    }
 }
 
 proc Buttons_Folder { frame } {
@@ -147,13 +165,12 @@ proc Buttons_Folder { frame } {
     set buttons(folderF) $frame
 
     # Menu for extra stuff
-    foreach M [concat [option get $frame menulist {}] \
-		      [option get $frame umenulist {}]] {
-	ButtonMenuInner $frame $M
+    foreach M [Widget_GetMenuBDef $frame] {
+	set menu [Widget_AddMenuBDef $frame $M {right padx 1}]
+	ButtonMenuInner $menu
     }
 
-    foreach b [concat [option get $frame buttonlist {}] \
-		      [option get $frame ubuttonlist {}]] {
+    foreach b [Widget_GetButDef $frame] {
 	if {$inc(style) == "none" && $b == "inc"} continue
 	Widget_AddButDef $frame $b
     }
@@ -165,42 +182,62 @@ proc Buttons_Message { frame } {
 
     # Menu for extra stuff
     # Loop through system and user-defined menus
-    foreach M [concat [option get $frame menulist {}] \
-		      [option get $frame umenulist {}]] {
-	set menub [ButtonMenuInner $frame $M]
+    foreach M [Widget_GetMenuBDef $frame] {
+	set menu [Widget_AddMenuBDef $frame $M {right padx 1}]
+	ButtonMenuInner $menu
 
 	# but only deal with system-defined groups
-	foreach g [option get $frame groups {}] {
-	    Buttons_GroupMenu $menub $g [option get $frame gm_$g {}]
-	    Buttons_GroupMenu $menub $g [option get $frame ugm_$g {}]
+	foreach g [Widget_GetGroupDef $frame] {
+	    Buttons_GroupMenu $menu $g [Widget_GetMenuGrDef $frame $g]
 	}
     }
 
-    foreach b [concat [option get $frame buttonlist {}] \
-		      [option get $frame ubuttonlist {}]] {
+    foreach b [Widget_GetButDef $frame] {
 	Widget_AddButDef $frame $b
     }
 
     # The group assignments associate buttons with states.
 
-    foreach g [option get $frame groups {}] {
-	Buttons_Group $frame $g [option get $frame g_$g {}]
-	Buttons_Group $frame $g [option get $frame ug_$g {}]
+    foreach g [Widget_GetGroupDef $frame] {
+	Buttons_Group $frame $g [Widget_GetButGrDef $frame $g]
     }
 }
-proc ButtonMenuInner { frame M } {
-    set menub [Widget_AddMenuBDef $frame $M {right padx 1}]
-    foreach e [concat [option get $menub entrylist {}] \
-		      [option get $menub uentrylist {}]] {
-	set l [option get $menub l_$e {}]
-	set c [option get $menub c_$e {}]
-	set v [option get $menub v_$e {}]
-	case [option get $menub t_$e {}] {
-	    default {Widget_AddMenuItem $menub $l $c}
-	    check   {Widget_CheckMenuItem $menub $l $c $v}
-	    radio   {Widget_RadioMenuItem $menub $l $c $v}
-	    separator {Widget_AddMenuSeparator $menub}
+proc ident { args } {
+    concat $args
+}
+# Ugh! - macro expand the variable name in
+# the context of the (original) caller of ButtonMenuInner
+# Allows variable references in the app-defaults file
+# The split-join trick is basically a no-op that gives the
+# TCL interpreter a chance to do variable expansion.  The
+# list and protected brackets are required to defend against
+# command strings ($c values) that contain semi-colons
+proc ButtonMenuInner { menu {level 1} } {
+    global pgp
+
+    foreach e [Widget_GetEntryDef $menu] {
+	set l [option get $menu l_$e {}]
+        if {$pgp(enabled) || ![string match "*PGP*" $l]} {
+	    set c [option get $menu c_$e {}]
+	    set v [option get $menu v_$e {}]
+	    set x [option get $menu x_$e {}]
+	    set v [uplevel $level [list subst $v]]	 ;# was ident
+	    set c [uplevel $level [list subst $c]]
+	    Exmh_Debug \"$l\" $c
+	    case [option get $menu t_$e {}] {
+		default {Widget_AddMenuItem $menu $l $c}
+		check   {Widget_CheckMenuItem $menu $l $c $v}
+		radio   {Widget_RadioMenuItem $menu $l $c $v $x}
+		cascade {
+		    set sub [option get $menu m_$e {}]
+		    if {[string length $sub] != 0} {
+			set submenu [Widget_CascadeMenuItem $menu $l $c $sub]
+			ButtonMenuInner $submenu [expr $level+1]
+		    }
+		}
+		separator {Widget_AddMenuSeparator $menu}
+	    }
 	}
     }
-    return $menub
+    return $menu
 }
