@@ -18,7 +18,7 @@ proc Exwin_Init {} {
 Contrained scrolling keeps the last line of text in a window stuck
 to the bottom of the window.  This can be set independently for
 the scan listing (FTOC)." {
-	{exwin(scrollbarSide) scrollbarSide {CHOICE right left} {Scrollbar side}
+	{exwin(scrollbarSide) scrollbarSide {CHOICE left right} {Scrollbar side}
 "Which side the scrollbars appear on.  This
 only takes effect after you restart exmh."}
 	{exwin(scrollSpeed) scrollSpeed 1 {Drag-Scroll speed}
@@ -36,6 +36,9 @@ constrain the scrolling of the folder table-of-contents."}
     {widgetText(scrollContext) textContextLines 2 "Scroll Context"
 "Scroll context is the number of lines that pages
 overlap when paging up and down through text."}
+    {widgetText(smoothScroll) textSmoothScroll ON "Smooth-scroll ends"
+"If set, the last partial page is scrolled line-by-line instead
+of jumping to the new location.  This helps you maintain context."}
     {widgetText(selectDelay) textSelectDelay 50 "Scroll/Select Time Constant"
 "When you drag a selection off the top or bottom of a text widget,
 the widget automatically scrolls and extends the selection.  This
@@ -49,7 +52,14 @@ once manually and they will always appear there.  However, if you
 use a virtual root window manager and run exmh in different \"rooms\"
 then a remembered placement might be in the wrong room.  You can nuke
 all the placement memory from the end of the .exmh-defaults file and
-always run exmh from the same room, or just disable this feature.."}
+always run exmh from the same room, or just disable this feature."}
+    {exwin(keepPlaces) keepPlaces ON {Remember within session}
+"With this enabled, exmh will remember window placements within a
+session.  This means that even if \"Remember window placement\" is
+OFF, if you place a window and dismiss it, exmh will remember its
+placement and re-use it for the rest of the current session.
+Disabling this option always allows your window manager to place
+windows."}
     }
     set exwin(ftextLinesSave) $exwin(ftextLines)
     trace variable exwin(ftextLines) w ExwinFixupFtextLines
@@ -101,25 +111,22 @@ proc Exwin_Layout {} {
 
     # Folder display (Ftoc)
     set exwin(ftext) [Widget_Text [Widget_Frame . ftoc Ftoc $fixed] \
-				$exwin(ftextLines) -cursor hand2 -wrap none]
+				$exwin(ftextLines)]
     Ftoc_Bindings $exwin(ftext)
     Ftoc_ColorConfigure $exwin(ftext)
 
     # Frame for faces, status, message buttons
     set mid     [Widget_Frame . mid Mid $fixed]
     Widget_SplitFrameR $mid Face Right
-    set face $mid.left
-    canvas $face.c -width 48 -height 48
-    set exwin(faceCanvas) $face.c
-    # default image
-    set exwin(faceFg) [option get $face.c foreground Foreground]
-    $face.c create bitmap 0 0 -anchor nw -bitmap @$exmh(library)/exmh.bitmap \
-	-foreground $exwin(faceFg)
-    pack append $face $face.c {top}
+    Faces_Create $mid.left
 
     # Status line + MsgID
     set right $mid.right
     Widget_SplitFrameV $right Status Mops
+
+    # FTOC/MSG boundary changer
+    ExwinFtocMsgBoundary $right.top
+
     set exwin(status) [Widget_Entry $right.top msg {right expand fill}]
     set statusConfig [option get . statusConfig StatusConfig]
     if {[string length $statusConfig] > 0} {
@@ -145,8 +152,181 @@ proc Exwin_Layout {} {
     # The <FocusIn> binding is for when the user has "focus-follows-mouse"
     # and moves the mouse into the main window from, say, the sedit window.
 
-    focus default $exwin(mtext)
-    bind . <FocusIn> [list focus $exwin(mtext)]
+    global tk_version
+    if {$tk_version < 4.0} {
+	focus default $exwin(mtext)
+	bind . <FocusIn> [list focus $exwin(mtext)]
+    } else {
+	focus $exwin(mtext)
+    }
+}
+proc ExwinFtocMsgBoundary {frame} {
+    global exwin
+    set c [canvas $frame.boundary -width 16 -height 15]
+    pack $frame.boundary -side right -fill y
+    set fg [option get . c_foreground {}]
+    set it [$c create poly 8 2  15 9  8 16  1 9 -fill $fg]
+    $c bind $it <ButtonPress-1> {ExwinFtocMsgScroll %W %x %y}
+    $c bind $it <B1-Motion> {ExwinFtocMsgMove %W %x %y}
+    $c bind $it <ButtonRelease-1> {ExwinFtocMsgStop %W %x %y}
+    set exwin(mode) {}
+}
+proc ExwinTopY {w y} {
+    # Find Y hit relative to toplevel window
+    set top [winfo toplevel $w]
+    while {[string compare $w $top] != 0} {
+	incr y [winfo y $w]
+	set w [winfo parent $w]
+    }
+    return $y
+}
+proc ExwinFtocMsgScroll {canvas x y} {
+    global exwin
+    set top [winfo toplevel $canvas]
+    if {$top == "."} {
+	set exwin(boundary) .boundary
+    } else {
+	set exwin(boundary) $top.boundary
+    }
+    set bg [option get . c_foreground {}]
+    frame $exwin(boundary) -width [winfo width $top] -height 2 -bg $bg
+    place $exwin(boundary) -y [ExwinTopY $canvas $y] -x 0 -anchor w
+    global fdisp
+    Exmh_Status "Adjust FTOC (and other) subwindow boundaries"
+
+    # Record Y coordinate of bottom of each subwindow
+    set exwin(yftoc) [ExwinTopY $exwin(ftext) [winfo height $exwin(ftext)]]
+    if [info exists fdisp(cache)] {
+	set exwin(yfcache) \
+	    [ExwinTopY $fdisp(cache) [winfo height $fdisp(cache)]]
+    } else {
+	catch {unset exwin(yfcache)}
+    }
+    if {!$fdisp(toplevel)} {
+	set exwin(yfdisp) \
+	    [ExwinTopY $fdisp(canvas) [winfo height $fdisp(canvas)]]
+    } else {
+	catch {unset exwin(yfdisp)}
+    }
+    set exwin(mode) ftoc
+}
+proc ExwinFtocMsgMove {canvas x y} {
+    global exwin
+    set ytop [ExwinTopY $canvas $y]
+    place $exwin(boundary) -y $ytop
+    switch $exwin(mode) {
+	ftoc {
+	    if {[info exists exwin(yfcache)] &&
+		$ytop <= $exwin(yfcache)} {	# Above FTOC window
+		set exwin(mode) fcache
+		Exmh_Status "Adjust Folder Cache boundary"
+	    } elseif {[info exists exwin(yfdisp)] &&
+		$ytop <= $exwin(yfdisp)} {	# Above Fcache window
+		set exwin(mode) fdisp
+		Exmh_Status "Adjust Folder Display boundary"
+	    }
+	}
+	fcache {
+	    if {[info exists exwin(yfdisp)] &&
+		$ytop <= $exwin(yfdisp)} {	# Above Fcache window
+		set exwin(mode) fdisp
+		Exmh_Status "Adjust Folder Display boundary"
+	    }
+	    if {$ytop >= $exwin(yftoc)} { # Below FTOC window
+		set exwin(mode) ftoc
+		Exmh_Status "Adjust FTOC boundary"
+	    }
+	}
+	fdisp {
+	    if {[info exists exwin(yfcache)] &&
+		$ytop >= $exwin(yfcache)} {	# Below Fcache window
+		set exwin(mode) fcache
+		Exmh_Status "Adjust Folder Cache boundary"
+	    } elseif {$ytop >= $exwin(yftoc)} { # Below FTOC window
+		set exwin(mode) ftoc
+		Exmh_Status "Adjust FTOC boundary"
+	    }
+	}
+    }
+}
+proc ExwinFtocMsgStop {canvas x y} {
+    global exwin fdisp
+    catch {destroy $exwin(boundary)}
+    # Deduce height of text line in FTOC
+    if ![info exists exwin(ftocLineHeight)] {
+	if [catch {ExwinLineHeight $exwin(ftext)} x] {
+	    Exmh_Status "Display a message before resizing" warn
+	    return
+	}
+	set exwin(ftocLineHeight) $x
+    }
+    switch $exwin(mode) {
+	ftoc {
+	    set dy [expr [ExwinTopY $canvas $y] - $exwin(yftoc)]
+	    set chunk $exwin(ftocLineHeight)
+	}
+	fcache {
+	    set dy [expr [ExwinTopY $canvas $y] - $exwin(yfcache)]
+	    set chunk [expr $fdisp(itemHeight) + $fdisp(ygap)]
+	}
+	fdisp {
+	    set dy [expr [ExwinTopY $canvas $y] - $exwin(yfdisp)]
+	    set chunk [expr $fdisp(itemHeight) + $fdisp(ygap)]
+	}
+    }
+    Exmh_Debug "dy $dy chunk $chunk"
+    set dl [expr int(round($dy / double($chunk)))]
+    if {$dl != 0} {
+	# The exwin(ftextLines) and fdisp(maxLines) are traced so
+	# the display updates when they change.
+	switch $exwin(mode) {
+	    ftoc {
+		set x [expr $exwin(ftextLines) + $dl]
+		if {$x <= 0} {set x 1}
+		set exwin(ftextLines) $x
+		set msg "Saving preference: $exwin(ftextLines) FTOC lines"
+		set var exwin(ftextLines)
+	    }
+	    fcache {
+		global fcache
+		set x [expr $fcache(lines) + $dl]
+		if {$x <= 0} {set x 1}
+		set fcache(lines) $x
+		set msg "Saving preference: $fcache(lines) Folder Cache lines"
+		set var fcache(lines)
+	    }
+	    fdisp {
+		set x [expr $fdisp(maxLines) + $dl]
+		if {$x <= 0} {set x 1}
+		set fdisp(maxLines) $x
+		set msg "Saving preference: $fdisp(maxLines) Folder Display lines"
+		set var fdisp(maxLines)
+	    }
+	}
+	# Let redisplay kick in
+	after 100 "
+	    Exmh_Status \"$msg...\"
+	    Preferences_Tweak $var
+	    Exmh_Status \"$msg...ok\"
+	"
+    } else {
+	Exmh_Status ok
+    }
+}
+proc ExwinLineHeight {w} {
+    set i 0
+    if {[scan [$w index @0,$i] %d top] != 1} {
+	error ExwinLineHeight
+    }
+    set limit [winfo height $w]
+    while {$i < $limit} {
+	incr i
+	scan [$w index @0,$i] %d next
+	if {$next != $top} {
+	    return $i
+	}
+    }
+    error "Cannot handle empty windows"
 }
 proc Exwin_FullFtoc {} {
     global exwin
@@ -220,6 +400,10 @@ proc Exwin_Toplevel { path name {class Dialog} {dismiss yes}} {
 	if {$ix < 0} {
 	    lappend exwin(toplevels) $t
 	}
+	set decor [option get $path clientDecoration ClientDecoration]
+	if {$decor == "none"} {
+	    wm transient $path
+	}
 	if {$dismiss == "yes"} {
 	    set f [Widget_Frame $t but Menubar {top fill}]
 	    Widget_AddBut $f quit "Dismiss" [list Exwin_Dismiss $path]
@@ -228,12 +412,24 @@ proc Exwin_Toplevel { path name {class Dialog} {dismiss yes}} {
     } else {
 	if {$state != "normal"} {
 	    catch {
-		wm geometry $path $exwin(geometry,$path)
+		if {$exwin(keepPlaces)} {
+		    wm geometry $path $exwin(geometry,$path)
+		    wm positionfrom $path user
+		} else {
+		    wm geometry $path {}
+		    wm positionfrom $path program
+		}
 		Exmh_Debug Exwin_Toplevel $path $exwin(geometry,$path)
 	    }
 	    wm deiconify $path
 	} else {
-	    catch {raise $path}
+	    catch {
+		if {! $exwin(keepPlaces)} {
+		    wm geometry $path {}
+		    wm positionfrom $path program
+		}
+		raise $path
+	    }
 	}
 	return 0
     }
@@ -251,17 +447,26 @@ proc Exwin_Dismiss { path {geo ok} } {
 	    catch {unset exwin(geometry,$path)}
 	}
     }
-    wm withdraw $path
+    if [info exists exwin(geometry,$path)] {
+	# Some window managers return geometry like
+	# 80x24+-1152+10
+	regsub -all {\+-} $exwin(geometry,$path) + exwin(geometry,$path)
+    }
     Exmh_Focus    
+    wm withdraw $path
+    update idletasks	;# Helps window dismiss
 }
 proc Exwin_ToplevelFocus { toplevel {default none} } {
     global exwin
     if {[string compare $default none] != 0} {
 	focus $default
     }
-    set exwin(focus,$toplevel) $default
-    bind $toplevel <FocusIn> {ExwinSetFocus %W}
-    bind $toplevel <FocusOut> {ExwinSaveFocus %W}
+    global tk_version
+    if {$tk_version < 4.0} {
+	set exwin(focus,$toplevel) $default
+	bind $toplevel <FocusIn> {ExwinSetFocus %W}
+	bind $toplevel <FocusOut> {ExwinSaveFocus %W}
+    }
 }
 proc ExwinSetFocus { w } {
     global exwin
