@@ -29,8 +29,8 @@ proc Fdisp_Init {} {
 
     Preferences_Add "Folder Display" \
 "These items affect the display of the labels in the folder display window.
-If you change key bindings on labels, you'll have to resize the window to
-force a redisplay because that's when the bindings are set." {
+If you change key bindings on labels, you'll have to toggle one of the other
+options to force a redisplay because that's when the bindings are set." {
 	{fdisp(maxLines) fdispLines 4 {Max fdisp rows}
 "The maximum number of rows of folder labels in
 the folder display.  If there are more folders than
@@ -41,6 +41,9 @@ toplevel window.  You can use the *Fltop.position Xresource
 to control its initial placement on the screen, and the
 *Fltop.Canvas.width and *Fltop.Canvas.height to control
 its size."}
+	{fdisp(oneCol) fdisp1Col OFF {Use one column}
+"Enabling this with a detached display aligns all the
+folders in one vertical column."}
 	{fdisp(popdownStyle) fdispPopdownStyle {CHOICE polygon rectangle} {Subfolder popdown}
 "The style of the sub folder popdown menu
 used to display subfolders." }
@@ -79,12 +82,14 @@ be implemented by triggering on <Enter> to other labels." }
     set fdisp(lastMaxLines) $fdisp(maxLines)
     trace variable fdisp(toplevel) w FdispFixupToplevel
     set fdisp(lastToplevel) $fdisp(toplevel)
-
-    set fdisp(poptop) -1
-
+    trace variable fdisp(oneCol) w FdispFixupRedisplay
+    trace variable fdisp(popdownStyle) w FdispFixupRedisplay
 }
+
+# Hooks for recreating the folder display when config changes
 proc FdispFixupMaxLines { args } {
     global exwin fdisp
+
     if [catch {expr {$fdisp(maxLines) * 2}}] {
 	set fdisp(maxLines) $fdisp(lastMaxLines)
 	return	;# bogus value
@@ -97,31 +102,43 @@ proc FdispFixupMaxLines { args } {
 	$fdisp(canvas) configure -height $h
     }
 }
+proc FdispFixupRedisplay { args } {
+    Fdisp_Redisplay
+}
 proc FdispFixupToplevel { args } {
-    global exwin fdisp tk_version
+    global fdisp
     if {$fdisp(toplevel) != $fdisp(lastToplevel)} {
-	if {$fdisp(toplevel)} {
-	    destroy $fdisp(frame)
-	    FdispMakeToplevel
-	    unset fdisp(frame)
-	} else {
+	if [info exists fdisp(topWidget)] {
 	    destroy $fdisp(topWidget)
-	    FdispMakeFrame
 	    unset fdisp(topWidget)
+	} else {
+	    destroy $fdisp(frame)
+	    unset fdisp(frame)
 	}
+	FdispMake
     }
     set fdisp(lastToplevel) $fdisp(toplevel)
 }
+
+# Make the folder display
+proc FdispMake {} {
+    global fdisp
+    if {$fdisp(toplevel)} {
+	FdispMakeToplevel
+    } else {
+	FdispMakeFrame
+    }
+}
+
+# Create folder list in a toplevel and make the canvas inside it
 proc FdispMakeToplevel { } {
-    global fdisp tk_version
+    global fdisp
     Exwin_Toplevel .fl "Folder list" Fltop nomenu
     set fdisp(topWidget) .fl
     wm minsize $fdisp(topWidget) 100 30
     wm protocol .fl WM_DELETE_WINDOW FdispDeleted
     FdispMakeCanvas $fdisp(topWidget)
-    if {$tk_version >= 4.0} {
-	focus $fdisp(canvas)
-    }
+    focus $fdisp(canvas)
     set icon [option get $fdisp(topWidget) iconposition IconPosition]
     catch {
 	Exwin_IconPosition $fdisp(topWidget) $icon
@@ -134,37 +151,41 @@ proc FdispMakeToplevel { } {
 	wm iconify $fdisp(topWidget)
     }
 }
+proc FdispDeleted {} {
+    wm iconify .fl
+    Exmh_Status "Folder display closed, not destroyed"
+}
 proc Fdisp_Checkpoint { varName } {
     # Add Xresources lines to $varName that save window size
     upvar $varName newstuff
-    global fdisp tk_version
+    global fdisp
     catch {
 	set can $fdisp(topWidget).can
 	set width [winfo width $can]
 	set height [winfo height $can]
-	set bd [lindex [$can config -borderwidth] 4]
-	if {$tk_version >= 4.0} {
-	    incr bd [$can cget -highlightthickness]
-	}
+	set bd [$can cget -borderwidth]
+	incr bd [$can cget -highlightthickness]
 	set width [expr $width - 2*$bd]
 	set height [expr $height - 2*$bd]
 	lappend newstuff "*Fltop.Canvas.height:\t$height"
 	lappend newstuff "*Fltop.Canvas.width:\t$width"
     }
 }
-proc FdispDeleted {} {
-    wm iconify .fl
-    Exmh_Status "Folder display closed, not destroyed"
-}
+
+# Create folder list in a frame and make the canvas inside it
 proc FdispMakeFrame { } {
     global fdisp
     set fdisp(frame) [Widget_Frame $fdisp(parent) f1 Frame]
     FdispMakeCanvas $fdisp(frame)
 }
+
+# Create the canvas for the folder display
 proc FdispMakeCanvas { frame } {
-    global fdisp tk_version exwin
-    set fdisp(canvas) [canvas $frame.can -bd 2 -relief raised ]
-    set s [scrollbar $frame.sv -command [list $fdisp(canvas) yview]]
+    global fdisp exwin
+    set fdisp(canvas) [canvas $frame.can -bd 2 -relief raised \
+	-highlightthickness 0]
+    set s [scrollbar $frame.sv -command [list $fdisp(canvas) yview] \
+	-highlightthickness 0]
     $fdisp(canvas) configure -yscrollcommand [list $s set]
 
     # Find out how big labels are
@@ -182,37 +203,31 @@ proc FdispMakeCanvas { frame } {
     set fdisp(charWidth) [expr {([lindex $size 2] - [lindex $size 0])/10}]
     $fdisp(canvas) delete $id
 
-    if {$tk_version < 4.0} {
-	$fdisp(canvas) configure -scrollincrement \
+    catch {
+	$fdisp(canvas) configure -yscrollincrement \
 	    [expr {$fdisp(itemHeight)+$fdisp(ygap)+1}]
-    } else {
-	catch {
-	    $fdisp(canvas) configure -yscrollincrement \
-		[expr {$fdisp(itemHeight)+$fdisp(ygap)+1}]
-	}
-	bindtags $fdisp(canvas) [list $fdisp(canvas) Command [winfo toplevel $fdisp(canvas)] all]
     }
-    if {! $fdisp(toplevel)} {
+
+    if {!$fdisp(toplevel)} {
 	set h [expr {$fdisp(maxLines)*($fdisp(itemHeight) + $fdisp(ygap)) + $fdisp(ygap)}]
 	$fdisp(canvas) configure -height $h
     }
+
+    bindtags $fdisp(canvas) \
+	[list $fdisp(canvas) Command [winfo toplevel $fdisp(canvas)] all]
     bind $fdisp(canvas) <2> {%W scan mark %x %y}
     bind $fdisp(canvas) <B2-Motion> {%W scan dragto %x %y}
-    if {! $fdisp(toplevel)} {
-	if [info exists fdisp(itemHeight)] {
-	    set h [expr {$fdisp(maxLines)*($fdisp(itemHeight) + $fdisp(ygap)) + $fdisp(ygap)}]
-	    $fdisp(canvas) configure -height $h
-	}
-    }
     bind $fdisp(canvas) <Configure> FdispCanvasConfigure
     pack $s -side $exwin(scrollbarSide) -fill y
     pack $fdisp(canvas) -side $exwin(scrollbarSide) -fill both -expand 1
+
     # fdisp popup color hack
     if {[tk colormodel $fdisp(canvas)] != "color"} {
 	if {! [regexp {black|white} $fdisp(c_popup)]} {
-	    set fdisp(c_popup) [lindex [$fdisp(canvas) config -bg] 4]
+	    set fdisp(c_popup) [$fdisp(canvas) cget -bg]
 	}
     }
+
     FdispDragAttach canvas
 }
 proc FdispFixupFont { args } {
@@ -255,12 +270,7 @@ proc Fdisp_Window { parent } {
     Widget_Frame $parent bogus Frame
 
     # The following creates fdisp(canvas), either in a toplevel or a frame
-    if {$fdisp(toplevel)} {
-	FdispMakeToplevel
-     } else {
-	FdispMakeFrame
-     }
-
+    FdispMake
 
     global fcache
     if $fcache(enabled) {
@@ -321,6 +331,9 @@ proc Fdisp_Layout { can folderSet {folder {}} {force 0} } {
 	return
     }
     set width [winfo width $canvas]
+    set bd [$canvas cget -borderwidth]
+    incr bd [$canvas cget -highlightthickness]
+    set width [expr $width - 2*$bd]
 
     if {! $force &&
 	($width == $fdisp(width,$can)) &&
@@ -337,12 +350,19 @@ proc Fdisp_Layout { can folderSet {folder {}} {force 0} } {
     set fdisp(fset,$can) $folderSet
 
     catch { $canvas delete all }
+    Fdisp_ClearSpecials $canvas
+
     if {$can != "cache"} {
+	FdispPopdownReset
 	Exmh_Status "Building folder display... $folder"
     }
     set fdisp(maxy,$can) [FdispLayoutInner $can $fdisp(xgap) $fdisp(ygap) \
 		    $width $folderSet $folder FdispBindLabel]
-    FdispSetCanvasSize $canvas $fdisp(maxy,$can)
+
+    set fdisp(bgid,$can) [$canvas create rect 0 0 0 0 \
+		-fill [$canvas cget -bg] -outline ""]
+    $canvas lower $fdisp(bgid,$can)
+    FdispSetCanvasSize $can $fdisp(maxy,$can)
     if {$can != "cache"} {
 	Exmh_Status ""
     }
@@ -352,22 +372,23 @@ proc Fdisp_Layout { can folderSet {folder {}} {force 0} } {
 	after 1 [list Fdisp_Layout $can $folderSet $folder]
     }
 }
-proc FdispLayoutInner { can x1 y1 width folderSet folder bindProc {skipSelf 0} {tag _notag_} } {
+proc FdispLayoutInner { can x1 y1 width folderSet folder bindProc {skipSelf no} {tag _notag_} } {
     global fdisp
     set canvas $fdisp($can)
     set maxy $fdisp(itemHeight)		;# Per row max item height
     set x $x1
     set y $y1
+    set iscache [string match cache $can]
     foreach f $folderSet {
 	# Determine label text for the folder
 	if {[string compare $f $folder] == 0} {
-	    if {[string compare $skipSelf "0"] != 0} {
+	    if [string match skipSelf $skipSelf] {
 		continue
 	    } else {
 		set text ".."
 	    }
 	} else {
-	    if {$can == "cache"} {
+	    if $iscache {
 		set text [Fcache_FolderName $f]
 	    } else {
 		set text [file tail $f]
@@ -378,7 +399,8 @@ proc FdispLayoutInner { can x1 y1 width folderSet folder bindProc {skipSelf 0} {
 	set bbox [$canvas bbox $id]
 	set twidth [expr [lindex $bbox 2]-[lindex $bbox 0]]
 	set theight [expr [lindex $bbox 3]-[lindex $bbox 1]]
-	if {$twidth + $fdisp(xgap)/2 + $x > $width} {
+	if {($twidth + $fdisp(xgap)/2 + $x > $width) ||
+		(($fdisp(oneCol) && !$iscache) && ($y > $y1 || $x > $x1))} {
 	    incr y [expr {$fdisp(ygap) + $maxy}]
 	    set x $x1
 	    set maxy $fdisp(itemHeight)		;# Per row max item height
@@ -395,7 +417,7 @@ proc FdispLayoutInner { can x1 y1 width folderSet folder bindProc {skipSelf 0} {
 	    set ftype goParent
 	} else {
 	    if [Flist_SubFolders $f] {
-		if {[string compare $can "cache"] == 0} {
+		if $iscache {
 		    # This supresses the drop-shadow in the cache display,
 		    # but also turns off the redisplay mode behavior...
 		    set ftype leaf
@@ -414,10 +436,10 @@ proc FdispLayoutInner { can x1 y1 width folderSet folder bindProc {skipSelf 0} {
 	    $bindProc $can $box $ftype $f
 	}
     }
-    return [expr {$y + $maxy}]
+    return [expr $y + $maxy]
 }
 proc Fdisp_Label { canvas f text } {
-    global fdisp folderInfo
+    global fdisp folderInfo fdispSpecial
     if [info exists folderInfo(bitmap,$f)] {
 	set special 0
 	if [info exists folderInfo(fg,$f)] {
@@ -438,8 +460,8 @@ proc Fdisp_Label { canvas f text } {
 	if {! $special} {
 	    $canvas addtag bitmap withtag $id
 	} else {
-	    lappend folderInfo(special,$canvas) $id
-	    set folderInfo(special,$canvas,$id) [list $fg $bg]
+	    lappend fdispSpecial($canvas) $id
+	    set fdispSpecial($canvas,$id) [list $fg $bg]
 	}
     } else {
 	set id [$canvas create text 0 0 -anchor nw \
@@ -448,18 +470,28 @@ proc Fdisp_Label { canvas f text } {
     return $id
 }
 proc Fdisp_FixupSpecials { canvas } {
-    global folderInfo
-    if ![info exists folderInfo(special,$canvas)] {
+    global fdispSpecial
+    if ![info exists fdispSpecial($canvas)] {
 	return
     }
-    foreach id $folderInfo(special,$canvas) {
-	if [info exists folderInfo(special,$canvas,$id)] {
-	    set fg [lindex $folderInfo(special,$canvas,$id) 0]
-	    set bg [lindex $folderInfo(special,$canvas,$id) 1]
+    foreach id $fdispSpecial($canvas) {
+	if [info exists fdispSpecial($canvas,$id)] {
+	    set fg [lindex $fdispSpecial($canvas,$id) 0]
+	    set bg [lindex $fdispSpecial($canvas,$id) 1]
 	    $canvas itemconfigure $id -background $bg -foreground $fg
 	}
     }
 }
+proc Fdisp_ClearSpecials { canvas } {
+    global fdispSpecial
+    if ![info exists fdispSpecial($canvas)] {
+	return
+    }
+    foreach id $fdispSpecial($canvas) {
+	unset fdispSpecial($canvas,$id)
+    }
+    unset fdispSpecial($canvas)
+}
 proc Fdisp_Box { canvas tid ftype {tag {}} } {
     # outline box.  I note that for variable width fonts,
     # the bbox is too long.  Oh well.
@@ -507,6 +539,9 @@ proc FdispBindLabel { can id ftype f } {
 		    [list Folder_TargetMove $f]
     $canvas bind $id <Shift-$fdisp(tarbutton)> \
 		    [list Folder_TargetCopy $f]
+    $canvas bind $id <Control-$fdisp(tarbutton)> \
+		    [list Folder_TargetClear $f]
+
 
     if {[string compare $ftype goParent] == 0} {
 	$canvas bind $id <$fdisp(navbutton)> \
@@ -540,27 +575,41 @@ proc FdispBindLabel { can id ftype f } {
 		$canvas bind $id <$fdisp(navbutton)> {}
 	    } else {
 		if {$fdisp(popdownRemove) == "navbutton"} {
-		    $canvas bind $id <$fdisp(navbutton)> \
-			[list FdispDisplayPopdown {} remove]
+		    $canvas bind $id <$fdisp(navbutton)> FdispPopdownRemove
 		} else {
 		    # Use enter on another leaf label to simulate Leave
 		    # of the popdown.  Cannot bind to <Leave> on the popdown
 		    # background because that triggers when you enter one
 		    # of its own labels.
-		    $canvas bind $id <Enter> \
-			[list FdispDisplayPopdown {} remove]
+		    $canvas bind $id <Enter> FdispPopdownRemove
 		}
 	    }
 	}
     }
 }
-proc FdispSetCanvasSize { canvas maxy } {
+proc FdispSetCanvasSize { can maxy {noshrink 0}} {
     global fdisp
 
-    set w [winfo width $canvas]
-    set h [expr {$maxy + $fdisp(ygap) + $fdisp(ygap)}]
+    set canvas $fdisp($can)
+    set w $fdisp(width,$can)
+
+    set height [winfo height $canvas]
+    set bd [$canvas cget -borderwidth]
+    incr bd [$canvas cget -highlightthickness]
+    set height [expr $height - 2*$bd]
+
+    set h [expr $maxy + $fdisp(ygap) + $fdisp(ygap)]
+    if {$height > $h} {set h $height}
+
+    #puts "SetSize $maxy->$h [lindex [$canvas cget -scrollregion] 3]"
+
+    if {$noshrink && [lindex [$canvas cget -scrollregion] 3] > $h} {
+	return
+    }
+
     $canvas configure -scrollregion [list 0 0 $w $h]
-    return $h
+    # adjust background to cover new scrollregion
+    $canvas coords $fdisp(bgid,$can) 0 0 $w $h
 }
 
 proc FdispUpdateMap { can folder id } {
@@ -618,7 +667,7 @@ proc FdispWhichLabel { can f } {
     # Figure out what label to highlight, handling nesting
     global fdisp mhProfile
 
-    if [FdispNotDotDot $can $f] {
+    if {"$can" == "cache" || [FdispNotDotDot $can $f]} {
 	return $f
     }
     while {[string compare $f "."] && [string compare $f "/"]} {
@@ -659,7 +708,7 @@ proc FdispNotDotDot { can f } {
     global fdisp
     set map [FdispGetMap $can $f]
     if {$map != {}} {
-	if [catch {lindex [$fdisp($can) itemconfigure $map -text] 4} l] {
+	if [catch {$fdisp($can) itemcget $map -text} l] {
 	    if [string compare $f ".."] {
 		return 1
 	    }
@@ -702,8 +751,11 @@ proc FdispHighlightCur { can f } {
     }
 }
 proc Fdisp_HighlightTarget { f } {
-    global fdisp
-    Fcache_Folder $f
+    global fdisp fcache
+
+    if $fcache(cacheTarget) {
+       Fcache_Folder $f
+    }
     foreach can {canvas cache} {
 	if [info exists fdisp($can)] {
 	    FdispHighlightTarget $can $f
@@ -732,7 +784,11 @@ proc FdispHighlightTarget { can f } {
 }
 
 proc Fdisp_HighlightUnseen { f } {
-    global fdisp
+    global fdisp fcache
+
+    if $fcache(cacheUnseen) {
+       Fcache_Folder $f
+    }
     foreach can {canvas cache} {
 	if [info exists fdisp($can)] {
 	    FdispHighlightUnseen $can $f
@@ -799,6 +855,9 @@ proc Fdisp_Lines { canvas labels } {
     set x $fdisp(xgap)
     set lines 1
     set width [winfo width $canvas]
+    set bd [$canvas cget -borderwidth]
+    incr bd [$canvas cget -highlightthickness]
+    set width [expr $width - 2*$bd]
     foreach folder $labels {
 	set f [Fcache_FolderName $folder]
 	set id [Fdisp_Label $canvas $f $f]
@@ -881,18 +940,14 @@ tlog-add .t "released on $dstw"
 proc FdispDragSelect {c x y wx wy} {
 	global fdisp
 
-	set wy [$c canvasy $wy]
-	set items [$c find overlapping $wx $wy $wx $wy]
+	set closest [$c find closest [$c canvasy $wx] [$c canvasy $wy]]
 
 	# Find what folder we're over
-	foreach item $items {
-		set tags [$c gettags $item]
-		set which [lsearch -glob $tags F*=*]
-		if {$which >= 0} {
-			set tag [lindex $tags $which]
-			regsub -- .*=(.*) $tag {\1} folder
-			break
-		}
+	set tags [$c gettags $closest]
+	set which [lsearch -glob $tags F*=*]
+	if {$which >= 0} {
+		set tag [lindex $tags $which]
+		regsub -- .*=(.*) $tag {\1} folder
 	}
 	if ![info exists folder] return
 
