@@ -49,6 +49,8 @@ proc Find_Setup {} {
 	if ![info exists find(all)] {
 	    set find(all) 0
 	}
+    } else {
+	$find(entry) delete 0 end
     }
     focus $find(entry)
 }
@@ -92,6 +94,8 @@ proc Find_It { {dir _default_} } {
 	global exwin
 	catch {wm geometry .find $exwin(geometry,$path)}
 	wm deiconify .find
+	Find_Setup
+	return
     } else {
 	catch {raise .find}
     }
@@ -138,13 +142,12 @@ proc Find_Inner { string dir start max matchProc {feedback yes} } {
     Exmh_Debug "Find_Inner line $L max $max wrap $find(wrap) wline $find(wrapLine)"
     if {$dir == "forw"} {
 	for { } {$L <= $max} {incr L} {
-	    set match [FindMatch $matchProc $L $string]
-	    if {$match < 0} {
-		break
-	    }
-	    if {$match} {
-		set find(wrap) 0
-		return 1	;# find(line) has been updated
+	    switch -- [FindMatch $matchProc $L $string] {
+		-1 { return 0 }
+		1  {
+		    set find(wrap) 0
+		    return 1	;# find(line) has been updated
+		}
 	    }
 	}
 	if {! $find(wrap)} {
@@ -155,19 +158,22 @@ proc Find_Inner { string dir start max matchProc {feedback yes} } {
 	}
 	set find(wrap) 0
 	for {set L 0} {$L < $find(line)} {incr L} {
-	    if {[FindMatch $matchProc $L $string] == 1} {
-		return 1
+	    switch -- [FindMatch $matchProc $L $string] {
+		-1 { return 0 }
+		1  {
+		    set find(wrap) 0
+		    return 1	;# find(line) has been updated
+		}
 	    }
 	}
     } else {
 	for { } {$L >= 1} {incr L -1} {
-	    set match [FindMatch $matchProc $L $string]
-	    if {$match < 0} {
-		break
-	    }
-	    if {$match} {
-		set find(wrap) 0
-		return 1
+	    switch -- [FindMatch $matchProc $L $string] {
+		-1 { return 0 }
+		1  {
+		    set find(wrap) 0
+		    return 1	;# find(line) has been updated
+		}
 	    }
 	}
 	if {! $find(wrap)} {
@@ -178,8 +184,12 @@ proc Find_Inner { string dir start max matchProc {feedback yes} } {
 	}
 	set find(wrap) 0
 	for {set L $max} {$L > $find(line)} {incr L -1} {
-	    if [FindMatch $matchProc $L $string] {
-		return 1
+	    switch -- [FindMatch $matchProc $L $string] {
+		-1 { return 0 }
+		1  {
+		    set find(wrap) 0
+		    return 1	;# find(line) has been updated
+		}
 	    }
 	}
     }
@@ -189,8 +199,8 @@ proc Find_Inner { string dir start max matchProc {feedback yes} } {
 proc FindMatch { hook L string } {
     global find
     if [catch {$hook $L $string} match] {
-	Exmh_Status $match
-	return 0
+	Exmh_Status "$match"
+	return -1
     }
     if {$match == 1} {
 	set find(line) $L
@@ -217,11 +227,7 @@ proc FindTextMatch {t L string} {
     } else {
 	set find(lastchar2) 0
     }
-    if [catch {regexp -nocase -indices $string $text match} hit] {
-	Exmh_Status $hit
-	return 0
-    }
-    if $hit {
+    if {[regexp -nocase -indices $string $text match]} {
 	global msg
 	set range [$t tag ranges sel]
 	if {$range != {}} {
