@@ -35,11 +35,17 @@ proc Flag_Init {} {
     Preferences_Resource flag(labelup) iconUpLabel {$flist(newMsgs) Unseen}
     Preferences_Resource flag(labeldown) iconDownLabel exmh
     Preferences_Resource flag(labelspool) iconSpoolLabel {$exmh(numUnInced) Spooled}
+    Preferences_Resource flag(iconupmask) iconUpMask flagup.mask
+    Preferences_Resource flag(icondownmask) iconDownMask flagdown.mask
+    Preferences_Resource flag(iconspoolmask) iconSpoolMask flagspool.mask
 
-    foreach i {iconup icondown iconspool} {
+    foreach i {iconup icondown iconspool iconupmask icondownmask iconspoolmask} {
 	if ![string match /* $flag($i)] {
 	    set flag($i) $exmh(library)/$flag($i)
 	}
+	if ![file exists $flag($i)] {
+	    set flag($i) 0
+	}
     }
     FlagInner down icondown labeldown
 }
@@ -66,6 +72,9 @@ proc FlagInner {state icon label} {
     global exmh flag
     if {$flag(state) != $state} {
 	wm iconbitmap . @$flag($icon)
+	if {$flag(${icon}mask) != 0} {
+	    wm iconmask . @$flag(${icon}mask)
+	}
 	set flag(state) $state
     }
     set l [uplevel #0 list $flag($label)]
@@ -73,6 +82,7 @@ proc FlagInner {state icon label} {
 	([string compare $l $flag(lastLabel)] == 0)} {
 	return
     }
+    wm title . $l
     wm iconname . $l
     set flag(lastLabel) $l
 
