@@ -14,6 +14,7 @@
 proc Folder_Init {} {
     global exmh argc argv mhProfile
     set exmh(target) {}		;# Name of target, for refile
+    set exmh(started) 0		;# For Folder_Change, the first time
     if {$argc > 0 && \
 	[file isdirectory $mhProfile(path)/[lindex $argv 0]]} then {
 	#scan named folder
@@ -77,7 +78,7 @@ proc FolderChange {f msgShowProc} {
 	    wm deiconify .
 	}
     }
-    if {[Ftoc_Changes "Change folder"] > 0} {
+    if {$exmh(started) && [Ftoc_Changes "Change folder"] > 0} {
 	# Need to reselect previous button here
 	return
     }
@@ -99,8 +100,14 @@ proc FolderChange {f msgShowProc} {
 	global mhProfile
 	set summary [Mh_Folder $f]	;# Set MH folder state
     } else {
+	if {$ftoc(folder) == {} && $exmh(started)} {
+	    # pseudo-display -> Checkpoint to set cur msg
+	    # startup -> don't checkpoint (clears cur sequence)
+	    Exmh_Debug Exmh_CheckPoint [time Exmh_CheckPoint]
+        }
 	set summary {}
     }
+    set exmh(started) 1
     global folderHook
     if [info exists folderHook(leave,$oldFolder)] {
 	$folderHook(leave,$oldFolder) $oldFolder leave
@@ -138,27 +145,30 @@ proc Folder_Target {f} {
 
     if ![file isdirectory $mhProfile(path)/$f] {
 	Exmh_Status "$mhProfile(path)/$f doesn't exist"
-	return
+	return 0
     }
     if {$exmh(folder) == $f} {
 	Exmh_Status "Target must be different than current" red
-	return
+	return 0
     }
     Fdisp_HighlightTarget $f
     set exmh(target) $f
     Exmh_Status "$f is target for moves"
+    return 1
 }
 proc Folder_TargetMove { f {moveProc Ftoc_MoveMark} } {
-    Folder_Target $f
-    Msg_Move $moveProc
+    if [Folder_Target $f] {
+	Msg_Move $moveProc
+    }
 }
 
 proc Folder_TargetCopy { f {copyProc Ftoc_CopyMark} } {
-    Folder_Target $f
-    Msg_Move $copyProc advance?
+    if [Folder_Target $f] {
+	Msg_Move $copyProc advance?
+    }
 }
 
-proc Folder_TargetClear {f} {
+proc Folder_TargetClear {{f {}}} {
     global exmh
 
     Fdisp_HighlightTarget ""
@@ -219,14 +229,14 @@ proc FolderCommit { rmmCommit moveCommit copyCommit } {
 }
 # Streamlined Commit called before Folder_Change
 proc Folder_CommitType { type } {
-    global ftoc
+    global ftoc exmh
     Exmh_Debug Folder_CommitType $type
     if {[string compare $type "Change folder"] == 0} {
 	Msg_CheckPoint
 	busy Ftoc_Commit Mh_Rmm Mh_Refile Mh_Copy
 	if $ftoc(autoPack) {
 	    Background_Wait	;# Let folder ops complete
-	    BgAction Pack Folder_Pack	;# Before packing
+	    BgAction Pack Mh_Pack $exmh(folder)	;# Before packing
 	}
     } else {
 	Folder_Commit
@@ -244,7 +254,7 @@ proc Folder_Purge { {folder {}} } {
     }
     exec touch $fn
     set now [file mtime $fn]
-    exec rm $fn
+    File_Delete $fn
 
     global mhProfile
     if ![info exists mhProfile(delprefix)] {
@@ -258,7 +268,7 @@ proc Folder_Purge { {folder {}} } {
     foreach f [glob -nocomplain $mhProfile(path)/$folder/$mhProfile(delprefix)*] {
 	if {[file mtime $f] + $purgesecs < $now} {
 	    Exmh_Debug Purge $f
-	    exec rm $f
+	    File_Delete $f
 	    incr n
 	}
     }
@@ -288,6 +298,8 @@ proc Folder_PurgeBg { {folderlist {}} } {
     while {[file exists [set fn [Env_Tmp]/exmh.[pid].purge.$uid]]} {
 	incr uid
     }
+    catch {File_Delete}	;# auto-load it
+
     set out [open $fn w]
     puts $out "wm withdraw ."
     puts $out "source $exmh(library)/folder.tcl"
@@ -296,6 +308,7 @@ proc Folder_PurgeBg { {folderlist {}} } {
     puts $out [list set mhProfile(path) $mhProfile(path)]
     puts $out "proc Exmh_Status { s } \{catch \{send \"[winfo name .]\" \[list Exmh_Status \$s]\}\}"
     puts $out "proc Exmh_Debug { args } {}"
+    puts $out [list proc File_Delete [info args File_Delete] [info body File_Delete]]
     foreach folder $folderlist {
 	puts $out "Folder_Purge $folder"
     }
