@@ -970,27 +970,35 @@ proc HMmap_reply {string} {
 # version of the library routine, until the bug is fixed, make sure we
 # over-ride the library version, and not the otherway around
 
-auto_load tkFocusOK
- proc tkFocusOK w {
-    set code [catch {$w cget -takefocus} value]
-    if {($code == 0) && ($value != "")} {
-    if {$value == 0} {
-        return 0
-    } elseif {$value == 1} {
-        return 1
-    } else {
-        set value [uplevel #0 $value $w]
-        if {$value != ""} {
-        return $value
-        }
+if {0} {
+    
+    # This bug is related to tabbing between embedded windows in a text
+    # widget, which is not important to exmh.  Defining this procedure
+    # can cause loops in the Tcl auto_load mechanism.
+
+    auto_load tkFocusOK
+     proc tkFocusOK w {
+	set code [catch {$w cget -takefocus} value]
+	if {($code == 0) && ($value != "")} {
+	if {$value == 0} {
+	    return 0
+	} elseif {$value == 1} {
+	    return 1
+	} else {
+	    set value [uplevel #0 $value $w]
+	    if {$value != ""} {
+	    return $value
+	    }
+	}
+	}
+	set code [catch {$w cget -state} value]
+	if {($code == 0) && ($value == "disabled")} {
+	return 0
+	}
+	regexp Key|Focus "[bind $w] [bind [winfo class $w]]"
     }
-    }
-    set code [catch {$w cget -state} value]
-    if {($code == 0) && ($value == "disabled")} {
-    return 0
-    }
-    regexp Key|Focus "[bind $w] [bind [winfo class $w]]"
 }
+
 # simple stuff to support interactive variable tracing
 # Module prefix is T
 #  - print value any time global variable changes
@@ -1109,7 +1117,7 @@ proc Deval {} {
 			C   {set ok 0}
 			?   {Dshow $level}
 			G	{
-				catch { uplevel #0 [llrange $line 1 end]} result
+				catch { uplevel #0 [lrange $line 1 end]} result
 				Stderr $result
 			}
 			W	{
