@@ -13,6 +13,13 @@
 set msg(maxsize) 50000
 
 proc Msg_Setup { t } {
+    # Tags to pre-allocate other important colors
+    global fdisp exmh
+    foreach level {error warn normal background} {
+	$t tag configure hdrlook=exmhColor$level -background $exmh(c_st_$level)
+    }
+    $t tag configure hdrlook=exmhColorPopup -background $fdisp(c_popup)
+    # Tags for header looks
     global msg
     foreach tagname $msg(tagnames) {
 	# Set up looks for different header lines
@@ -22,7 +29,6 @@ proc Msg_Setup { t } {
 	    if [catch {
 		eval {$t tag configure hdrlook=$tagname} $rval
 	    } err] {
-		Exmh_Debug tag configure hdrlook=$tagname $rval: $err
 		unset msg(tag,$tagname)
 	    }
 	}
@@ -39,11 +45,18 @@ proc Msg_Setup { t } {
     TextButton_Init $t
 
     $t tag raise sel
+
+    # HACK To cache font for graphics part separator.
+    catch {
+	label $t.fontlabel6 -font -*-*-*-*-*-*-6-*-*-*-*-*-iso8859-*
+	label $t.fontlabel8 -font -*-*-*-*-*-*-8-*-*-*-*-*-iso8859-*
+    }
+
 }
 
 proc Msg_Redisplay { draft } {
     global msg mhProfile exmh msg
-    if {[string compare $draft $mhProfile(path)/$exmh(folder)/$msg(id)] == 0} {
+    if {[string compare $draft $msg(path)] == 0} {
 	set msg(dpy) {}
 	MsgShow $msg(id)
     }
@@ -52,16 +65,21 @@ proc Msg_Redisplay { draft } {
 
 proc MsgShow { msgid } {
     # Display the current message in a text widget
-    global msg exwin exmh mhProfile mimeHdr
+    global msg exwin exmh mhProfile mimeHdr mime
 
     if {$msg(dpy) == $msgid} {
 	return
     }
+    Html_Stop $exwin(mtext)
     Label_Message $exmh(folder):$msgid
+    Audit "Show $exmh(folder) $msgid"
     if [MsgShowInText $exwin(mtext) $mhProfile(path)/$exmh(folder)/$msgid] {
 	MsgSeen $msgid
-	set msg(dpy) $msgid
+	if {!$mime(stop)} {
+	    set msg(dpy) $msgid
+	}
 	set msg(curclear) 0
+	set mime(stop) 0
 	update idletasks	;# Faces display can be slow
 	Face_Show [MsgParseFrom $mimeHdr(0=1,hdr,from)] $mimeHdr(0=1,hdr,x-face)
 	foreach cmd [info commands Hook_MsgShow*] {
@@ -74,6 +92,7 @@ proc MsgShowInText { win file } {
     global mhProfile msg mimeHdr exmh msg mhProfile mime
     $win configure -state normal
     $win delete 0.0 end
+    $win mark set insert 1.0
 
     if [info exists mhProfile(exmhshowproc)] {
 	Exmh_Debug MsgShowInText $mhProfile(exmhshowproc) $file
@@ -91,9 +110,11 @@ proc MsgShowInText { win file } {
     set mimeHdr($part=$subpart,hdr,subject) {}
     set mimeHdr($part=$subpart,hdr,x-face) {}
     set mimeHdr($part=$subpart,fullHeaders) $mime(fullHeaders)
+    set mimeHdr($part=$subpart,yview) 1.0
 
     set mimeHdr($part,decode) 1
     set mimeHdr($part,file) $fileName
+    set mimeHdr($part,rawfile) $file
     set mimeHdr($part,color) [lindex [$win configure -background] 4]
     set mimeHdr($part,type) message/rfc822
     set mimeHdr($part,encoding) 7bit
@@ -110,6 +131,9 @@ proc MsgShowInText { win file } {
     }
 
     set partTag [MimeLabel $part part]
+    set defaultTag [MimeLabel $part=1 part]
+    $win tag configure $defaultTag -background [$win cget -background] \
+	-foreground [$win cget -foreground]
     MimeSetPartVars desc displayedPart $win $part $partTag
     if {$mimeHdr($part,numParts) > 0} {
 	$win config -cursor watch
@@ -118,22 +142,26 @@ proc MsgShowInText { win file } {
     }
     $win config -cursor xterm
     MimeInsertSeparator $win $part 6
+    Widget_TextPad $win $mimeHdr(0=1,yview)
+    $win yview $mimeHdr(0=1,yview)
 
     catch {unset mimeLastPoint}
     catch {unset mimeTagStack}
 
-    Exmh_Status $desc
+    Exmh_Status "$desc"
 
     $win configure -state disabled
     return 1
 }
 
-proc MsgParseFrom { fromline } {
+proc MsgParseFrom { fromline {setaddr setaddr} } {
     set line [string trim $fromline]
     if [regsub {\(.*\)} $line {} newline] {
 	set line $newline
     }
-    if [regexp {[^ 	"]*@[^ 	"]*} $line token] {
+    if [regexp {<.*@.*>} $line token] {
+	set token [string trim $token <>]
+    } elseif [regexp {[^ 	"]*@[^ 	"]*} $line token] {
 	set token [string trim $token <>]
     } else {
 	if [regexp {<.*>} $line token] {
@@ -145,9 +173,11 @@ proc MsgParseFrom { fromline } {
 	    }
 	}
     }
-    # Link to alias interface
-    global address
-    set address $token
+    if {[string compare $setaddr "setaddr"] == 0} {
+	# Link to alias interface
+	global address
+	set address $token
+    }
     return $token
 }
 
