@@ -2,11 +2,11 @@
 # pgp.tcl
 #	PGP 2.6 support for exmh.
 #	Orginally contributed by Allan Downey
-#	Updated by Stefan Monnier, Anders Klemets and William Sproule
+#	Updated by Stefan Monnier, Anders Klemets, William Sproule,
+#	Chris Garrigues and Ben Escoto
 #
 
 # future:
-# - somehow arrange for "insert @" to use the decoded part...
 # - rewrite PGP decrypt, to deal with keys, mime stuff, etc...
 # - encrypt pgp parts, similar to the automatic quote-printable
 # - split big functions
@@ -19,6 +19,47 @@
 # to avoid auto-loading this whole file.
 
 # $Log: pgp.tcl,v $
+# Revision 1.1  1998/05/05 17:55:37  welch
+# Initial revision
+#
+# Revision 1.1  1998/05/05 17:42:59  welch
+# Initial revision
+#
+# Revision 1.16  1998/01/22  00:47:01  bwelch
+#     Fixed Pgp_Setup to use /dev/tty instead of /dev/console
+#
+# Revision 1.15  1997/12/22  20:53:46  bwelch
+# File_Delete
+#
+# Revision 1.14  1997/07/25  17:14:09  bwelch
+# Trapped error messages from PGP xterms.
+#
+# Revision 1.13  1997/07/12  23:06:28  bwelch
+#     Added expecttk support for PGP handling.
+#     Fixed recursive failures with some PGP messages.
+#
+# Revision 1.12  1997/06/03  18:37:00  bwelch
+# Added doinc flag to Inc_Presort
+# Major cleanup
+#
+# Revision 1.11  1997/01/25  06:21:16  bwelch
+# Added Quote module support
+#
+# Revision 1.10  1997/01/25  05:34:08  bwelch
+#     Added Pgp_GetTextAttributes
+# Changed display of PGP messages
+#
+# Revision 1.9  1996/12/21  00:58:08  bwelch
+# Fixed PGP menu to support both email and www key fetching.
+# Guard against missing signature parts in multipart/signed
+#
+# Revision 1.8  1996/12/02  21:11:11  bwelch
+# Moved New Key button to be inside the help documents.
+#
+# Revision 1.7  1996/12/01  20:16:47  bwelch
+# Added mutipart/security support.
+# (Ben Escoto and Chris Garrigues)
+#
 # Revision 1.6  1996/03/22  18:44:34  bwelch
 # Changed graphic part separator size in PGP from 5 to 6 to avoid
 #     downloading a new font for this case.
@@ -54,15 +95,12 @@
 proc Pgp_Setup {  } {
     global pgp env
 
-    if {[info exists pgp(path)] && \
-	    ([string length [string trim $pgp(path)]] > 0) && \
-	    ([lsearch -exact [split $env(PATH) :] $pgp(path)] < 0)} {
-	set env(PATH) $pgp(path):$env(PATH)
-    }
+    PgpSetPath
 
     foreach path [split $env(PATH) ":"] {
 	if [file executable "$path/pgp"] {
 	    set pgpexec "$path/pgp"
+	    break
 	}
     }
     
@@ -83,12 +121,25 @@ I can do anything for you."
     exec touch "$env(PGPPATH)/config.txt"
 
     # make the key pair and self sign it
-    exec xterm -title "PGP Setup" -e sh -c {
-	cd ${PGPPATH}
-	rm -f pubring.bak
+    if [catch {
+	exec xterm -title "PGP Setup" -e sh -c {
+	    cd ${PGPPATH}
+	    rm -f pubring.bak
+	    pgp -kg
+	    rm -f pubring.bak
+	    pgp +verbose=0 +force=on -ks "" -u ""
+	} >& /dev/tty
+    } error] {
+	Misc_DisplayText "PGP Setup Error" \
+"An error occurred while trying to generate your key:
+$error
+please try these commands at your unix shell
+to generate your key and self sign it:
 	pgp -kg
-	rm -f pubring.bak
-	pgp +verbose=0 +force=on -ks "" -u ""
+	pgp +verbose=0 +force=on -ks \"\" -u \"\"
+Then restart exmh to enable its PGP support.
+"
+        return
     }
 
     if {![file exists "$env(PGPPATH)/pubring.bak"]} {
@@ -106,12 +157,14 @@ I can do anything for you."
     # send the key to the keyservers
     PgpExec_GetKeys [lindex [lindex $pgp(privatekeys) 0] 0] $tmpfile
     Misc_Send $pgp(keyserver) ADD $tmpfile "content-type: application/pgp; format=keys-only"
-    exec rm -f $tmpfile
+    File_Delete $tmpfile
 }
 
 proc Pgp_Help {  } {
+    Help PGP	;# Make Key button is embedded in the HTML
+}
+proc Pgp_HelpOld {} {
     global exmh
-
     set label "Help about setting up PGP"
     if [Exwin_Toplevel .pgphelp "PGP Help" PgpHelp] {
 	Widget_Label .pgphelp.but label {left fill} -text $label
@@ -183,7 +236,7 @@ proc Pgp_Decrypt {  } {
     close $mess
 
     Mh_Rename $outfile $file
-    exec rm -f $pgpfile
+    File_Delete $pgpfile
 
     set msg(dpy) {}
     MsgChange $msg(id)
@@ -200,7 +253,7 @@ proc Pgp_ExmhEncrypt {  } {
 
     Exmh_Status "pgp -e $exmh(folder)/$msg(id)"
 
-    Pgp_Process $file $tmpfile "pgp-action: encrypt; recipients=[lindex $pgp(myname) 0]"
+    Pgp_Process $file $tmpfile "pgp-action: PGP/MIME-encrypt; recipients=[lindex $pgp(myname) 0]"
     Mh_Rename $tmpfile $file
 
     set msg(dpy) {}
@@ -238,6 +291,9 @@ proc Pgp_SeditEncrypt { action draft t } {
 	SeditMsg $t "PGP Not enabled"
 	return
     }
+    if {"$action" == "none"} {
+        return
+    }
     set pgpaction "Pgp-Action: $action"
     if $pgp(rfc822) {
 	append pgpaction "; rfc822=on"
@@ -286,6 +342,7 @@ proc Pgp_Process { srcfile dstfile {pgpaction {}} } {
 
     set orig [open $srcfile r]
 
+    Exmh_Debug Pgp_Process
     # get the header of the draft and split it into mime and non-mime headers
     set mailheaders [Misc_Segregate line \
 	  {[regexp $miscRE(mimeheaders) $line]} [Misc_GetHeader $orig]]
@@ -296,8 +353,6 @@ proc Pgp_Process { srcfile dstfile {pgpaction {}} } {
     # add the default pgp-action header and mime headers
     if {$pgpaction != {}} {
 	set mailheaders [linsert $mailheaders 0 $pgpaction]
-    } else {
-	lappend mailheaders "pgp-action: none"
     }
     if {[lsearch -glob $mimeheaders "content-type:*"] < 0} {
 	lappend mimeheaders "content-type: text/plain; charset=us-ascii"
@@ -323,8 +378,10 @@ proc Pgp_Process { srcfile dstfile {pgpaction {}} } {
 	}
     }
 
+
     # setup default params
-    set typeparams "; x-action=$action"
+    regsub {.*-} $action {} xaction
+    set typeparams "; x-action=$xaction"
     
     # setup rfc822
     if [info exists actionparam(rfc822)] {
@@ -373,7 +430,9 @@ proc Pgp_Process { srcfile dstfile {pgpaction {}} } {
     # write the message to be enrypted
     set msgfile [Mime_TempFile "msg"]
     set msg [open $msgfile w 0600]
-    foreach line $pgpheaders { puts $msg [Pgp_FixHeader $line] }
+    if ![regexp -nocase {^pgp-} $action] {
+	foreach line $pgpheaders { puts $msg [Pgp_FixHeader $line] }
+    }
     puts $msg ""
     puts -nonewline $msg [read $orig]
     close $orig
@@ -381,15 +440,29 @@ proc Pgp_Process { srcfile dstfile {pgpaction {}} } {
 
     set pgpfile [Mime_TempFile "pgp"]
     if [catch {
-	switch -exact $action {
-	    encrypt	{ PgpExec_Encrypt $msgfile $pgpfile $ids }
-	    signbinary	{ PgpExec_Sign $msgfile $pgpfile $originator 0 }
-	    signclear	{ PgpExec_Sign $msgfile $pgpfile $originator 1 }
-	    encryptsign	{ PgpExec_EncryptSign $msgfile $pgpfile $originator $ids }
+	switch $action {
+	    application/pgp-encrypt	{ 
+		PgpExec_Encrypt $msgfile $pgpfile $ids }
+	    application/pgp-signbinary	{
+		PgpExec_Sign $msgfile $pgpfile $originator 0 }
+	    application/pgp-signclear	{ 
+		PgpExec_Sign $msgfile $pgpfile $originator 1 }
+	    application/pgp-encryptsign	{
+		PgpExec_EncryptSign $msgfile $pgpfile $originator $ids }
+	    pgp/mime-signclear { 
+		PgpExec_SignPM $msgfile $pgpfile $originator }
+	    pgp/mime-encryptsign { 
+		PgpExec_EncryptSign $msgfile $pgpfile $originator $ids}
+	    pgp/mime-encrypt	{ PgpExec_Encrypt $msgfile $pgpfile $ids }
+	    pgp-signclear { 
+		PgpExec_Sign $msgfile $pgpfile $originator 1}
+	    pgp-encryptsign { 
+		PgpExec_EncryptSign $msgfile $pgpfile $originator $ids}
+	    pgp-encrypt	{ PgpExec_Encrypt $msgfile $pgpfile $ids }
 	    default	{ error "<PGP> unknown action $action" }
 	}
     } err] {
-	exec rm -f $msgfile
+	File_Delete $msgfile
 	error $err
     }
 
@@ -398,25 +471,258 @@ proc Pgp_Process { srcfile dstfile {pgpaction {}} } {
 	    [regexp -nocase $miscRE(true) $actionparam(localaction)]} { 
 	set mailheaders {}
     }
-    lappend mailheaders \
-	    "mime-version: 1.0" \
-	    "content-type: application/pgp; format=mime$typeparams" \
-	    "content-transfer-encoding: 7bit"
+	
+    switch -regexp $action {
+        application/pgp- { 
+            lappend mailheaders \
+                "mime-version: 1.0" \
+                "content-type: application/pgp; format=mime$typeparams" \
+                "content-transfer-encoding: 7bit"
 
-    # write out the new mail file
+            # write out the new mail file
+            set dst [open $dstfile w 0600]
+            foreach line $mailheaders { puts $dst [Pgp_FixHeader $line] }
+            puts $dst ""
+            set msg [open $pgpfile r]
+            puts -nonewline $dst [read $msg]
+            close $msg
+            close $dst
+        }
+        pgp/mime- { 
+            Pgp_ProcessPM $dstfile $pgpfile $mailheaders $action $msgfile
+        }
+        pgp- { 
+            Pgp_ProcessPlain $dstfile $pgpfile $mailheaders $action $msgfile
+        }
+    }
+    
+    
+    File_Delete $msgfile $pgpfile
+}
+
+proc Pgp_ProcessPM {dstfile pgpfile mailheaders action plainfile} {
+
+    set boundary [Mime_MakeBoundary P]
+
+    # Put in specified headers.  
+    lappend mailheaders "mime-version: 1.0"
+    if [regexp {encrypt} $action] {
+	lappend mailheaders "content-type: multipart/encrypted; boundary=\"$boundary\";\n\t protocol=\"application/pgp-encrypted\""
+    } else {
+	lappend mailheaders "content-type: multipart/signed; boundary=\"$boundary\";\n\t micalg=pgp-md5; protocol=\"application/pgp-signature\""
+    }
+    lappend mailheaders "content-transfer-encoding: 7bit"
+
+    # Write file
     set dst [open $dstfile w 0600]
+    set pgpIO [open $pgpfile r]
+
     foreach line $mailheaders { puts $dst [Pgp_FixHeader $line] }
     puts $dst ""
-    set msg [open $pgpfile r]
-    puts -nonewline $dst [read $msg]
-    close $msg
-    close $dst
+    puts $dst "--$boundary"
+    if [regexp {encrypt} $action] {
+	puts $dst "Content-Type: application/pgp-encrypted"
+	puts $dst ""
+	puts $dst "Version: 1"
+	puts $dst ""
+	puts $dst "--$boundary"
+	puts $dst "Content-Type: application/octet-stream"
+	puts $dst ""
+	puts $dst [read $pgpIO]
+	puts $dst "--$boundary--"
+    } else {
+	set plain [open $plainfile r]
+	puts $dst [read $plain]
+	close $plain
+	puts $dst "--$boundary"
+	puts $dst "Content-Type: application/pgp-signature"
+	puts $dst ""
+	puts $dst [read $pgpIO]
+	puts $dst "--$boundary--"
+    }
 
-    exec rm -f $msgfile $pgpfile
+    close $dst
+    close $pgpIO
+
+}
+
+proc Pgp_ProcessPlain {dstfile pgpfile mailheaders action plainfile} {
+
+    set dst [open $dstfile w 0600]
+    set pgpIO [open $pgpfile r]
+
+    foreach line $mailheaders { puts $dst [Pgp_FixHeader $line] }
+    puts $dst ""
+
+    puts $dst [read $pgpIO]
+
+    close $dst
+    close $pgpIO
+}
+
+proc PgpExec_SignPM { in out sigkey } {
+
+    PgpExec [list -stab $in -u [lindex $sigkey 0] -o $out] output $sigkey
+
+    if {![file exists $out] && [file exists "$out.asc"]} {
+	exec mv "$out.asc" $out
+    }
+    if {![file exists $out]} {
+	error "PGP refused to generate the signed text:\n$output"
+    } else {
+	return {}
+    }
+}
+
+proc MimeShowMultipartSignedPgp {tkw part} {
+    global mimeHdr mime pgp
+
+    if {$pgp(enabled)} {
+
+	if {![info exists mimeHdr($part,pgpdecode)]} {
+	    if {($pgp(showinline) == "all") ||
+		($pgp(showinline) == "signed")} {
+		set mimeHdr($part,pgpdecode) 1
+	    } else { set mimeHdr($part,pgpdecode) 0 }
+	}
+
+	MimeMenuAdd $part checkbutton \
+	    -label "Check signature with PGP" \
+	    -command [list busy MimeRedisplayPart $tkw $part] \
+	    -variable mimeHdr($part,pgpdecode)
+
+	if $mimeHdr($part,pgpdecode) {
+	    if [info exists mimeHdr($part=2,file)] {
+		PgpExec [list $mimeHdr($part=2,file) [GetSignedText \
+							  $tkw $part]] msg
+    
+		Pgp_InterpretOutput $msg pgpresult
+		Pgp_DisplayMsg $tkw $part pgpresult
+    
+		set mimeHdr($part=1,color) \
+		    [MimeDarkerColor $tkw $mimeHdr($part,color)]
+	    } else {
+		error "Missing signature"
+	    }
+	} else {
+	    $tkw insert insert \
+		"PGP signed message - the signature hasn't been checked\n"
+	    TextButton $tkw "Check signature" \
+		"$mimeHdr($part,menu) invoke {Check signature with PGP} \
+                 \n$tkw config -cursor xterm"
+	    $tkw insert insert "\n"
+	    MimeInsertSeparator $tkw $part 6
+	    set mimeHdr($part=1,color) $mimeHdr($part,color)
+	}
+
+    } else { set mimeHdr($part=1,color) $mimeHdr($part,color) }
+
+    MimeShowPart $tkw $part=1 [MimeLabel $part part] 1
+}
+
+
+proc MimeShowMultipartEncryptedPgp {tkw part} {
+    global mimeHdr pgp exmh
+
+    if {!$pgp(enabled)} {
+	Mime_ShowDefault $tkw $part
+	return
+    }
+
+    if {![info exists mimeHdr($part,pgpdecode)]} {
+	if {$pgp(showinline) == "all"} {
+	    set mimeHdr($part,pgpdecode) 1
+	} else { set mimeHdr($part,pgpdecode) 0 }
+	
+	MimeMenuAdd $part checkbutton \
+	    -label "Decrypt with PGP" \
+	    -command [list busy MimeRedisplayPart $tkw $part] \
+	    -variable mimeHdr($part,pgpdecode)
+    }  
+
+    if {!$mimeHdr($part,pgpdecode)} {
+	TextButton $tkw "Decrypt message" \
+	    "$mimeHdr($part,menu) invoke {Decrypt with PGP} \
+             \n$tkw config -cursor xterm"
+	$tkw insert insert "\n"
+	Mime_ShowDefault $tkw $part
+	return
+    }
+	
+    set tmpfile [Mime_TempFile "decrypt"]
+
+    # Decide whether or not to use expect
+    if {$pgp(keeppass) && [info exists exmh(expectk)] && $pgp(useexpectk)} {
+	PgpExec_DecryptExpect $mimeHdr($part=2,file) $tmpfile msg
+    } else {
+	# Assume only recipient is primary secret key
+	# Use expect to avoid this behavior
+	set recipients [Misc_Map elem {string trim $elem} \
+             [split [string range [lindex $pgp(myname) 0] 2 end] ","]]
+    
+	PgpExec_Decrypt $mimeHdr($part=2,file) $tmpfile msg $recipients
+    }
+
+    Pgp_InterpretOutput $msg pgpresult
+    Pgp_DisplayMsg $tkw $part pgpresult
+
+    set DarkerColor [MimeDarkerColor $tkw $mimeHdr($part,color)]
+    
+    # The following three lines would show the 
+    # application/pgp-encrypted mime section.
+
+    # set mimeHdr($part=1,color) $DarkerColor
+    # MimeShowPart $tkw $part=1 [MimeLabel $part part] 1
+    # MimeInsertSeparator $tkw $part 5
+    
+    set fileIO [open $tmpfile r]
+    set mimeHdr($part=2,color) $DarkerColor
+    set mimeHdr($part=2,numParts) [MimeParseSingle $tkw $part=2 $fileIO]
+    close $fileIO
+    
+    MimeShowPart $tkw $part=2=1 [MimeLabel $part part] 1
+}
+
+proc GetSignedText {tkw part} {
+    global mimeHdr
+
+    set fileIO [open $mimeHdr($part,file) r]
+    set boundary $mimeHdr($part,param,boundary)
+
+    # Prolog
+    while {([set numBytes [gets $fileIO line]] >= 0) &&
+	   ([string compare --$boundary $line] != 0) &&
+	   ([string compare --$boundary-- $line] != 0)} {}
+
+    set tmpFilename [Mime_TempFile $part=1]
+    set tmpFile [open $tmpFilename w 0600]
+
+    # Body
+    # Must not include the newline right before the boundary, that is
+    # considered part of the boundary, not the subpart. (rfc1521?)
+
+    # Also, LF should be converted to CRLF for checking.
+    
+    if {([set numBytes [gets $fileIO line]] >= 0) &&
+	([string compare --$boundary $line] != 0) &&
+	([string compare --$boundary-- $line] != 0)} {
+	puts -nonewline $tmpFile $line
+	while {([set numBytes [gets $fileIO line]] >= 0) &&
+	       ([string compare --$boundary $line] != 0) &&
+	       ([string compare --$boundary-- $line] != 0)} {
+	    puts $tmpFile "\r"
+	    puts -nonewline $tmpFile $line 
+	}
+    }
+ 
+    close $fileIO
+    close $tmpFile
+    
+return $tmpFilename
 }
 
 proc Pgp_ShowMessage { tkw part } {
-    global mimeHdr mime pgp miscRE
+    global mimeHdr mime pgp miscRE exmh
 
     set in [open $mimeHdr($part,file) r]
     gets $in firstLine
@@ -433,11 +739,14 @@ proc Pgp_ShowMessage { tkw part } {
     }
     set format $mimeHdr($part,param,format)
 
+    Exmh_Debug Pgp_ShowMessage format $format $part
+
     # the action pgp performed
     if {"$format" != "keys-only"} {
 	if {![info exists mimeHdr($part,param,x-action)]} {
 	    if [regexp $miscRE(beginpgpclear) $firstLine] {
 		set action signclear
+		set mimeHdr($part,param,x-action) signclear
 	    } else {
 		set action encryptsign
 	    }
@@ -480,46 +789,50 @@ proc Pgp_ShowMessage { tkw part } {
 	    set tmpfile [Mime_TempFile "decrypt"]
 	    
 	    if [regexp "encrypt" $action] {
-		PgpExec_Decrypt $mimeHdr($part,file) $tmpfile msg $recipients
+		
+		# Decide whether or not to use expect
+		if {$pgp(keeppass) && [info exists exmh(expectk)] \
+			&& $pgp(useexpectk)} {
+		    Exmh_Debug "Using expect"
+		    PgpExec_DecryptExpect $mimeHdr($part,file) \
+			$tmpfile msg
+		} else {
+		    PgpExec_Decrypt $mimeHdr($part,file) \
+			$tmpfile msg $recipients
+		}
 	    } else {
 		PgpExec [list $mimeHdr($part,file) -o $tmpfile] msg
 	    }
-	    
-	    set msg [string trim $msg " \t\n"]
-	    if {![file exists $tmpfile]} {
-		set msg "PGP failed to decrypt the file:\n$msg"
-	    }
-	    if {[regexp {sign} $action] && \
-		    [regexp -nocase {key id ([0-9a-f]+) } $msg {} keyid]} {
-		MimeMenuAdd $part command \
-			-label "Query keyserver for key $keyid" \
-			-command "Misc_Send {$pgp(keyserver)} {GET 0x$keyid}"
-		TextButton $tkw "Query keyserver" "Misc_Send {$pgp(keyserver)} {GET 0x$keyid}"
-		$tkw insert insert "\n"
-	    }
-	    if {($msg != {}) && ($msg != "child process exited abnormally")} {
-		$tkw insert insert "$msg\n"
-		MimeInsertSeparator $tkw $part 6
-	    }
-	    if [catch {open $tmpfile r} fileIO] {
-		return
-	    }
-	    exec rm -f $tmpfile
-	    if {$format == "mime"} {
+
+	    Pgp_InterpretOutput $msg pgpresult
+	    Pgp_DisplayMsg $tkw $part pgpresult
+
+	    if {$pgpresult(ok)} {
+		if [catch {set fileIO [open $tmpfile r]} err] {
+		    Exmh_Debug "Error: $err"
+		    return
+		}
+		File_Delete $tmpfile
+    
 		if {![info exists mimeHdr($part,numParts)]} {
-		    set mimeHdr($part,numParts) [MimeParseSingle $tkw $part $fileIO]
-		    set mimeHdr($part=1,color) [MimeDarkerColor $tkw $mimeHdr($part,color)]
+		    Exmh_Debug MimeParseSingle $part
+		    set mimeHdr($part,numParts) \
+			[MimeParseSingle $tkw $part $fileIO]
+		    set mimeHdr($part=1,color) \
+			[MimeDarkerColor $tkw $mimeHdr($part,color)]
 		}
 		MimeShowPart $tkw $part=1 [MimeLabel $part part] 1
-	    } else {
-		$tkw insert insert "[read $fileIO]"
+		MimeClose $fileIO
 	    }
-	    MimeClose $fileIO
+	    
 	} else {
 	    if {$action == "signclear"} {
-		$tkw insert insert "  the signature hasn't been checked "
+		$tkw insert insert \
+		    "PGP signed message - the signature hasn't been checked\n"
 		if $pgp(enabled) {
-		    TextButton $tkw "check sig" [list $mimeHdr($part,menu) invoke "$pgp(menutext,$action) with pgp..."]
+		    TextButton $tkw "Check signature" \
+		    [list $mimeHdr($part,menu) invoke \
+			 "$pgp(menutext,$action) with pgp..."]
 		}
 		$tkw insert insert "\n"
 		MimeInsertSeparator $tkw $part 6
@@ -532,7 +845,7 @@ proc Pgp_ShowMessage { tkw part } {
 		    set tmpfile "$mimeHdr($part,file).msg"
 		    Misc_StringFile $msg $tmpfile
 		    set fileIO [open $tmpfile r]
-		    exec rm -f $tmpfile
+		    File_Delete $tmpfile
 		    if {![info exists mimeHdr($part,numParts)]} {
 			set mimeHdr($part,numParts) [MimeParseSingle $tkw $part $fileIO]
 			set mimeHdr($part=1,color) [MimeDarkerColor $tkw $mimeHdr($part,color)]
@@ -543,6 +856,10 @@ proc Pgp_ShowMessage { tkw part } {
 		    $tkw insert insert $msg
 		}
 	    } else {
+		TextButton $tkw "Show with PGP" \
+		    [list $mimeHdr($part,menu) invoke \
+			 "$pgp(menutext,$action) with pgp..."]
+		$tkw insert insert "\n"
 		Mime_ShowDefault $tkw $part
 	    }
 	}
@@ -590,7 +907,49 @@ proc Pgp_InsertKeys { draft t } {
 	    }
 	    
 	    SeditInsertFile $draft $t $tmpfile 1 7bit {application/pgp; format=keys-only} "keys of [lindex $key 1]"
-	    exec rm -f $tmpfile
+	    File_Delete $tmpfile
 	}
     }
 }
+
+proc Pgp_GetTextAttributes { summary } {
+    global pgp
+
+    switch $summary {
+	GoodSignatureUntrusted {return $pgp(GoodUntrustedSig)}
+	GoodSignatureTrusted   {return $pgp(GoodTrustedSig)}
+	BadSignatureTrusted    {return $pgp(BadSig)}
+	BadSignatureUntrusted  {return $pgp(BadSig)}
+        default                {return $pgp(OtherMsg)}
+    }
+}
+
+proc Pgp_DisplayMsg { tkw part pgpresultvar } {
+    global pgp
+    upvar $pgpresultvar pgpresult
+
+    Exmh_Debug "Pgp_DisplayMsg: $pgpresult(msg)"
+    if {[info exists pgpresult(keyid)]} {
+	MimeMenuAdd $part command \
+	    -label "Query keyserver for key $pgpresult(keyid)" \
+	    -command "PgpQueryKey $pgpresult(keyid)"
+	if {[regexp "PublicMissing" $pgpresult(summary)]} {
+	    TextButton $tkw "Query keyserver" \
+		"PgpQueryKey $pgpresult(keyid)"
+	}
+	$tkw insert insert "\n"
+    }
+
+    set rval [Pgp_GetTextAttributes $pgpresult(summary)]
+    if {$rval != {}} {
+	if [catch {eval {$tkw tag configure PgpResults} $rval} err] {
+	    Exmh_Debug tag configure PgpResults $rval: $err
+	    $tkw insert insert "$pgpresult(msg)\n"
+	} else {
+	    $tkw insert insert "$pgpresult(msg)\n" PgpResults
+	}
+    } else {
+	$tkw insert insert "$pgpresult(msg)\n"
+    }
+    MimeInsertSeparator $tkw $part 6
+}
