@@ -145,8 +145,9 @@ proc Message_Apply { {body {}} } {
 
     if {$print(cancel)} {
 	if {$print(cleanup)} {
-	    catch {exec rm $file}
+	    File_Delete $file
 	}
+	Exmh_Focus
 	return
     }
 
@@ -167,8 +168,9 @@ proc Message_Apply { {body {}} } {
 	Msg_Redisplay $msg(path)
     }
     if {$print(cleanup)} {
-	catch {exec rm $file}
+	File_Delete $file
     }
+    Exmh_Focus
 }
 proc Message_ApplyCancel {} {
     global print exwin
