@@ -13,76 +13,125 @@
 
 #### Display folder contents
 
-proc Scan_Folder {F} {
-    Exmh_Debug Scan_Folder $F
-    Exmh_Debug Scan_Folder [time [list ScanFolder $F]]
+proc Scan_Folder {F {adjustDisplay 1}} {
+    Exmh_Debug Scan_Folder [time [list ScanFolder $F $adjustDisplay]]
 }
-proc ScanFolder {F} {
-    global mhProfile flist ftoc
-    set cacheFile $mhProfile(path)/$F/.xmhcache
+proc ScanFolder {F adjustDisplay} {
+    global mhProfile flist ftoc exwin exmh
 
-    set ftoc(displayValid) 0
-    set ftoc(displayDirty) 0
-    # Scan the folder, being clever in two different ways, if possible.
-    # 1) if we have flist, then we know how many new messages there
-    # are, so we just scan those and merge that info into our cached
-    # scan output
-    # 2) if there are no new messages, just read the scan cache
-    # 3) if there have been folder operations that invalidated the cache,
-    # then do a full scan.
+    if {[string compare $F $ftoc(folder)] == 0} {
+	Exmh_Debug Updating $F
+	set update 1	;# Need to check for new messages.
+	set sameF 1	;# Same folder as before
+	ScanAddLineInit
+    } else {
+	set sameF 0
+	set cacheFile $mhProfile(path)/$F/.xmhcache
+	if [catch {open $cacheFile} input] {
+	    # No cache, scan last N messages
+	    Exmh_Status "Limit scan $F last:$ftoc(scanSize) - Rescan?" warn
+	    set input  [open "|$mhProfile(scan-proc) +$F last:$ftoc(scanSize) \
+			    -width $ftoc(scanWidth)"]
+	    set ftoc(displayDirty) 1
+	    set update 0
+	} else {
+	    Exmh_Debug "loading .xmhcache"
+	    set ftoc(displayDirty) 0
+	    set update 1	;# Need to check for new messages.
+	}
+	ScanAddLineReset $F
+	ScanAddLines [read $input]
+	catch {close $input}
+    }
 
-    set U [Flist_NumUnseen $F]
-    if {$U > 0} {
-	set overlap 5
-	set numNewMsgs [expr $U+$overlap]
-	if {$numNewMsgs > $overlap} {
-	    set scanio [open "|scan +$F last:$numNewMsgs -width 100"]
-	    Exmh_Status "scanning new msgs in $F ..." blue
-	    if [ScanMerge $F $scanio $cacheFile] {
-		set ftoc(displayValid) 1
+    if {$update} {
+	# Add new messages to cached information
+	# Scan last message (again) plus any new messages
+	if {! $sameF} {
+	    Ftoc_Reset [Widget_TextEnd $exwin(ftext)] {} $F
+	}
+	set id [Ftoc_MsgNumber [Ftoc_FindMsg {} last]]
+	if [catch {
+	    Exmh_Debug Scanning new messages
+	    set input [open "|$mhProfile(scan-proc) +$F $id-last \
+				-width $ftoc(scanWidth)"]
+	    set check [gets $input]
+	    set new [read $input]
+	    close $input
+	} err] {
+	    # The last message no longer exists
+	    Exmh_Debug No last msg $id: $err
+        } else {
+	    set id2 [Ftoc_MsgNumberRaw $check]
+	    if {$id2 == $id} {
+		# Last message still matches: add the new lines
+		ScanAddLines $new
 		set ftoc(displayDirty) 1
-		Exmh_Status $F
-		return
+		set update 0	;# OK
+	    } else {
+		Exmh_Debug "My last $id != $id2"
+	    }
+	}
+	if {$update} {
+	    # Something went wrong: rescan
+	    if {[Ftoc_Changes "Scan Update Failed"] == 0} {
+	    Exmh_Status "scan +$F last:$ftoc(scanSize)"
+		Background_Wait
+		set input  [open "|$mhProfile(scan-proc) +$F last:$ftoc(scanSize)
+			    -width $ftoc(scanWidth)"]
+		set ftoc(displayDirty) 1
+		ScanAddLineReset $F
+		ScanAddLines [read $input]
+		catch {close $input}
 	    }
 	}
     }
-    # Inc_PresortFinish will call Scan_Folder with pending changes
-    # on the assumption that ScanMerge will always work.
-    # In case it doesn't, we bail out here.  Other calls to Scan_Folder
-    # make their own check of Ftoc_Changes
-    if [Ftoc_Changes "Scan Overwrite" noauto] {
-	Exmh_Status ""
-	return
-    }
-
-    if { ! [Scan_CacheValid $F]} {
-	Exmh_Status "rescanning $F ..." red
-	Scan_IO $F [open "|scan +$F -width 100"]
-	set ftoc(displayDirty) 1
+    ScanAddLineCleanup $F
+    if {! $sameF} {
+	Msg_Reset [Widget_TextEnd $exwin(ftext)] $F
     } else {
-	Exmh_Status "scan cache hit for $F ..." blue
-	Scan_IO $F [open $cacheFile]
-	set ftoc(displayDirty) 0
+	Ftoc_Update [Widget_TextEnd $exwin(ftext)] $F
     }
-    Exmh_Status $F
-    Label_Folder $F
     set ftoc(displayValid) 1
+    if {$adjustDisplay} {
+	Ftoc_Yview end
+    }
+    if {$update} {
+	Flist_ForgetUnseen $F
+    }
+    Ftoc_ShowUnseen $F
+    return
 }
 proc Scan_FolderForce {} {
-    global exmh mhProfile
+    global exmh mhProfile ftoc
     set F $exmh(folder)
     set cacheFile $mhProfile(path)/$F/.xmhcache
-    Exmh_Debug Scan_FolderForce
-    Msg_CheckPoint		;# save current sequence information
     if {$F == ""} {
 	Exmh_Status "No current folder" red
     } else {
 	if {! [Ftoc_Changes Rescan]} {
+	    Background_Wait
+	    Label_Folder $F
 	    Exmh_Status "rescanning $F ..."
-	    Scan_IO $F [open "|scan +$F -width 100"]
+	    Scan_IO $F [open "|$mhProfile(scan-proc) +$F -width $ftoc(scanWidth)"]
+	    set ftoc(displayValid) 1
+	    set ftoc(displayDirty) 1
+	    Ftoc_Yview end
+	    Flist_ForgetUnseen $F
+	    Ftoc_ShowUnseen $F
+	    Exmh_Status ok
 	}
     }
 }
+proc Scan_FolderUpdate { f } {
+    global ftoc
+
+    if !$ftoc(displayValid) {
+        return                  ;#  don't update pseudo-displays
+    }
+    Label_Folder $f
+    Scan_Folder $f 0
+}
 proc Scan_Iterate { incout lineVar body } {
     upvar $lineVar line
     foreach line [split $incout \n] {
@@ -96,16 +145,21 @@ proc Scan_Iterate { incout lineVar body } {
 }
 
 proc Scan_Inc {folder incOutput} {
+    global exwin ftoc
     # Append output of an Inc to the scan display
     ScanAddLineInit
     Scan_Iterate $incOutput l {
 	ScanAddLine $l
     }
     ScanAddLineCleanup $folder
-    Ftoc_ShowUnseen $folder
-    global exwin ftoc
-    Ftoc_Update [expr [lindex [split [$exwin(ftext) index end] .] 0]-1] $folder
+    Ftoc_Update [Widget_TextEnd $exwin(ftext)] $folder
     set ftoc(displayDirty) 1
+    if {$ftoc(showNew)} {
+	Ftoc_Yview end
+    }
+    # Don't forget unseen here, just find recently added unseen messages
+    Ftoc_ShowUnseen $folder
+    Label_Folder $folder
 }
 proc Scan_IO {folder scanIO } {
     Exmh_Debug Scan_IO [time [list ScanIO $folder $scanIO]]
@@ -119,118 +173,10 @@ proc ScanIO {folder scanIO } {
 	close $scanIO
     } err] {
 	Exmh_Status $err red
+	catch {close $scanIO}
     }
     ScanAddLineCleanup $folder
-    Ftoc_ShowUnseen $folder
-    Msg_Reset [expr [lindex [split [$exwin(ftext) index end] .] 0]-1] $folder
-}
-proc ScanMerge {folder scanIO cacheFile} {
-    global ftoc		;# In bed with ftoc display here
-    set ftoc(mergeok) 0
-
-    if [catch {
-	if {$ftoc(folder) == $folder} {
-	    ScanMergeCache $folder $scanIO
-	} else {
-	    ScanMergeCache $folder $scanIO $cacheFile
-	}
-    } err] {
-	Exmh_Error "ScanMerge \n$err"
-    }
-    return $ftoc(mergeok)
-}
-proc ScanMergeCache { folder scanIO {cacheFile {}} } {
-    global ftoc
-    if [catch {
-	set scan [split [read $scanIO] \n]
-	close $scanIO
-    } err] {
-	Exmh_Debug ScanMergeCache: $err
-	return 0
-    }
-    if {$cacheFile != {}} {
-	if [catch {open $cacheFile} cacheIO] {
-	    Exmh_Debug ScanMergeCache: $cacheIO
-	    return 0
-	}
-	# Read in cache and the scan output of the last several messages
-	set rawCache [read $cacheIO]
-	set cache [split $rawCache \n]
-	close $cacheIO
-	set displayCache 0
-    } else {
-	# Use the display as the cache, not the file
-	set displayCache 1
-    }
-
-    # Match up the scan output with the last lines in the cache
-    set sync 0
-    set scanIndex 0
-    set scanLine [lindex $scan $scanIndex]
-    set scanMsgNum [Ftoc_MsgNumberRaw $scanLine]
-    if {$displayCache} {
-	set nlines $ftoc(numMsgs)
-    } else {
-	# Split adds extra blank element, hence the -2 here
-	set nlines [expr [llength $cache]-2]
-    }
-    for {set L $nlines} {$L >= 0} {incr L -1} {
-	if {$displayCache} {
-	    set cacheMsgNum [Ftoc_MsgNumber $L]
-	} else {
-	    set line [lindex $cache $L]
-	    set cacheMsgNum [Ftoc_MsgNumberRaw $line]
-	}
-	if {$cacheMsgNum == $scanMsgNum} {
-	    incr L
-	    incr scanIndex
-	    set sync 1
-	    break
-	}
-    }
-    if {! $sync} {
-	return 0
-    }
-    for {} {$L <= $nlines} {incr L ; incr scanIndex} {
-	set scanLine [lindex $scan $scanIndex]
-	set scanMsgNum [Ftoc_MsgNumberRaw $scanLine]
-	if {$displayCache} {
-	    set cacheMsgNum [Ftoc_MsgNumber $L]
-	} else {
-	    set line [lindex $cache $L]
-	    set cacheMsgNum [Ftoc_MsgNumberRaw $line]
-	}
-	if {$cacheMsgNum != $scanMsgNum} {
-	    return 0
-	}
-    }
-    # Fill in the display if things seem ok
-    if {$displayCache} {
-	ScanAddLineInit
-    } else {
-	ScanAddLineReset $folder
-	ScanAddLines $rawCache
-    }
-    set scanLength [llength $scan]
-    for {} {$scanIndex < $scanLength} {incr scanIndex} {
-	set scanLine [lindex $scan $scanIndex]
-	if {$scanLine == {}} {
-	    continue
-	}
-	ScanAddLine $scanLine
-    }
-    ScanAddLineCleanup $folder
-    Ftoc_ShowUnseen $folder
-    global exwin
-    if {$displayCache} {
-	Ftoc_Update [expr [lindex [split [$exwin(ftext) index end] .] 0]-1] \
-		    $folder
-    } else {
-	Msg_Reset [expr [lindex [split [$exwin(ftext) index end] .] 0]-1] \
-		  $folder
-    }
-    set ftoc(mergeok) 1
-    return 1
+    Msg_Reset [Widget_TextEnd $exwin(ftext)] $folder
 }
 
 proc ScanAddLineInit {} {
@@ -245,6 +191,7 @@ proc ScanAddLineReset { folder } {
     }
     ScanAddLineInit
     $exwin(ftext) delete 0.0 end
+    update idletasks
 }
 proc ScanAddLine { line } {
     global exwin
@@ -262,7 +209,6 @@ proc ScanAddLineCleanup { folder } {
     }
     set ftoc(folder) $folder
     $exwin(ftext) configure -state disabled
-    WidgetTextYview $exwin(ftext) end
 }
 proc Scan_ProjectSelection { ids } {
     global ftoc exwin
@@ -303,21 +249,42 @@ proc Scan_CacheUpdate {} {
     if {$folder == {}} {
 	return
     }
-    if {!$ftoc(displayValid) || !$ftoc(displayDirty)} {
+    if !$ftoc(displayDirty) {
 	return
     }
     set cacheFile $mhProfile(path)/$folder/.xmhcache
-    if [catch {
+
+#
+# Display is invalid but changes (deletes) still must be reflected in cache. 
+# A full rescan is the penalty you have to pay for deleting messages inside 
+# this thing.
+#
+    if !$ftoc(displayValid) {
+	set curLine [Ftoc_ClearCurrent]			;# Clear +
+        if [file writable $cacheFile] {
+            set scancmd [list exec $mhProfile(scan-proc) \
+                               +$folder -width $ftoc(scanWidth) > $cacheFile]
+            if [catch $scancmd err] {
+                Exmh_Status "failed to rescan folder $folder: $err" warn
+            }
+        }
+	Ftoc_Change [Ftoc_MsgNumber $curLine] $curLine	;# Restore it
+    } elseif [catch {
 	set cacheIO [open $cacheFile w]
 	set curLine [Ftoc_ClearCurrent]			;# Clear +
-	set display [$exwin(ftext) get 1.0 end]
+	global tk_version
+	if {$tk_version < 4.0} {
+	    set display [$exwin(ftext) get 1.0 end]
+	} else {
+	    set display [$exwin(ftext) get 1.0 "end -1 char"]
+	}
 	Ftoc_Change [Ftoc_MsgNumber $curLine] $curLine	;# Restore it
 	puts $cacheIO $display nonewline
 	close $cacheIO
 	set ftoc(displayDirty) 0
-	Exmh_Debug Scan_CacheUpdate OK $folder
     } err] {
 	Exmh_Debug Scan_CacheUpdate error $err
+	catch {close $cacheIO}
     }
 }
 
@@ -334,7 +301,6 @@ proc Scan_Move { folder scanlinesR new } {
 	Exmh_Debug Scan_Move cannot handle scan format
 	return
     }
-    Exmh_Debug Scan_Move $folder
     set len [string length $prefix]
     set fmt [format "%%%dd%%s" $len]
     set cacheIO [open $cacheFile a]
@@ -351,3 +317,31 @@ proc Scan_Move { folder scanlinesR new } {
     }
     close $cacheIO
 }
+proc Scan_AllFolders { {force 0} } {
+    global flist mhProfile ftoc wish
+    if [catch {open [Env_Tmp]/scancmds w} out] {
+	Exmh_Status "Scan_AllFolders $out"
+	return
+    }
+    set ctx [Env_Tmp]/scanctx
+    puts $out "wm withdraw ."
+    set myname [winfo name .]
+    puts $out "catch \{ exec touch $ctx.\[pid\] \}"
+    puts $out "set env(MHCONTEXT) $ctx.\[pid\]"
+    foreach f $flist(allfolders) {
+	if {$force || ! [Scan_CacheValid $f]} {
+	    puts $out "catch \{send $myname \{Exmh_Status \"scan +$f\"\}\}"
+	    puts $out "catch {
+		set out \[open $mhProfile(path)/$f/.xmhcache w $mhProfile(msg-protect)\]
+		exec $mhProfile(scan-proc) +$f -width $ftoc(scanWidth) >@\$out
+		close \$out
+	    }"
+	}
+    }
+    puts $out "catch \{send $myname \{Exmh_Status \"scans completed\"\}\}"
+    puts $out "exec rm $ctx.\[pid\]"
+    puts $out exit
+    close $out
+    Exmh_Status "wish -f [Env_Tmp]/scancmds &" blue
+    exec $wish -f [Env_Tmp]/scancmds &
+}
