@@ -91,6 +91,8 @@ proc SeditComposedKeyBindings {} {
     set composedkey(~N) "\xD1"
     set composedkey(aa) "\xE5"
     set composedkey(AA) "\xC5"
+    set composedkey(*a) "\xE5"
+    set composedkey(*A) "\xC5"
     set composedkey(\"\") "\xA8"
     set composedkey(,,) "\xB8"
     set composedkey(!!) "\xA1"
