@@ -9,16 +9,19 @@
 # control laws. This software is made available AS IS, and Xerox Corporation
 # makes no warranty about the software, its performance or its conformity to
 # any specification.
+#
+#
 
 proc SeditWhom { draft f t } {
     global tk_version sedit
     set parent [file root $f]
-    if {[catch {destroy $parent.whom}] == 0} {
+    if {[winfo exists $parent.whom]} {
+	destroy $parent.whom
 	return
     }
     # Do an unformatted save so Mh_Whom gets the right info
     set format $sedit($t,format)
-    set sedit($t,format) 0
+    set sedit($t,format) Never
     if [catch {
 	SeditSave $draft $t
 	SeditDirty $t
@@ -45,12 +48,13 @@ proc SeditWhom { draft f t } {
     }
 }
 proc SeditSign { draft t {f ~/.signature} } {
+    global sedit
     if {[catch {glob $f} sig] || [string length $f]==0} {
 	return
     }
     set exec 0
     if [file executable $sig] {
-	set sig "|$sig"
+	set sig "|$sig $sedit($t,isigc) $sedit($t,isigf) $draft"
 	set exec 1
     } else {
 	if ![file readable $sig] {
@@ -68,15 +72,15 @@ proc SeditSign { draft t {f ~/.signature} } {
 	}
 	if {!$sedit($t,multi)} {
 	    if ($sedit(sigDashes)) {
-		$t insert end "\n-- \n"
+		$t insert $sedit(sigPosition) "\n-- \n"
 	    } else {
-		$t insert end \n
+		$t insert $sedit(sigPosition) \n\n
 	    }
 	    # check for 8bit characters in the signature
 	    if $8bit {
 		set sedit($t,8bit) 1
 	    }
-	    $t insert end $signature
+	    $t insert $sedit(sigPosition) $signature
 	} else {
 	    set type text/plain
 	    if $8bit {
@@ -97,8 +101,39 @@ proc SeditSign { draft t {f ~/.signature} } {
 	}
     }
 }
+proc SeditSignIntelligent { draft t {f ~/.signature} } {
+    global sedit intelligentSign
+    global mhProfile
+
+    set tmp_fmt $sedit($t,format)
+    set sedit($t,format) Never
+    set saveokay [SeditSave $draft $t {} 0]
+    set sedit($t,format) $tmp_fmt
+
+    set cmd {exec whom $draft}
+
+    if {!$saveokay || [catch {eval $cmd} rcpts]} {
+	Exmh_Status "Error finding recipients; using default signature $f"
+    } else {
+	regsub -all " at " $rcpts "@" rcpts
+	regsub -all "(^|\n)(\[^@\n\]+(\n|\$))+" $rcpts "\\1" rcpts
+
+	foreach domn $intelligentSign(domain) {
+	    regsub -all "(^|\n)(\[^\n\]+$domn\[^\n\]*(\n|\$))+" $rcpts {} rcpts
+	}
+
+	if {[regexp {[a-zA-Z0-9]} $rcpts]} {
+	    set f $intelligentSign(external)
+	    Exmh_Status "Using external signature $f"
+	} else {
+	    set f $intelligentSign(internal)
+	    Exmh_Status "Using internal signature $f"
+	}
+    }
+    SeditSign $draft $t $f
+}
 proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain} {desc {}}} {
-    global sedit mime
+    global sedit mime quote
     if {$newpart < 0} {
 	return
     }
@@ -118,23 +153,26 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 	}
 	set cmd ""
 	set uuname [file tail $file]
+	if ![regexp name= $type] {
+	    append type " ; name=\"$uuname\""
+	}
 	switch -- $encoding {
-	    base64 {   append cmd "| $mime(encode) -b " }
-	    quoted-printable {   append cmd "| $mime(encode) -q " }
+	    base64 {   lappend cmd | $mime(encode) -b }
+	    quoted-printable {   lappend cmd | $mime(encode) -q }
 	    none {set encoding {}}
-	    x-uuencode {   append cmd "| uuencode [file tail $file] " }
+	    x-uuencode {   lappend cmd | uuencode $uuname }
 	}
 	if {[string length $cmd] == 0} {
 	    set in [open $file]
 	} else {
-	    append cmd " < $file"
+	    lappend cmd < $file
 	    Exmh_Status $cmd
 	    set in [open $cmd r]
 	}
 	if [$t compare insert <= hlimit] {
 	    $t mark set insert "hlimit +1c"
 	}
-	if {$file == "@"} {
+	if {$file == $quote(filename)} {
 	    set inheaders 1
 	    set quoted 0
 	    while {[gets $in line] > -1} {
@@ -148,7 +186,6 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 			if {$sedit($t,quote) < 0} {
 			    set sedit($t,quote) 1
 			}
-			Exmh_Debug Quote @
 		    }
 		    if {[string length $line] == 0} {
 			set inheaders 0
@@ -157,7 +194,7 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 			    if [catch {open $tfile w} out] {
 				$t insert insert "Error: $out"
 			    } else {
-				puts $out [read $in]
+				puts -nonewline $out [read $in]
 				close $out
 				close $in
 				if [catch {open "|$mime(encode) -q -u < $tfile"} in] {
@@ -188,6 +225,15 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 		if {$encoding != {}} {
 		    $t insert $mark "Content-Transfer-Encoding: $encoding\n"
 		}
+		if {$mime(eudora)} {
+		    if $mime(dosname) {
+			set filename [Mime_EudoraFilename $file]
+		    } else {
+			set filename [file tail $file]
+		    }
+		    $t insert $mark \
+		    "Content-Disposition: attachment; filename=\"$filename\"\n"
+		}
 		$t insert $mark \n
 	    } else {
 		set mark insert
@@ -197,7 +243,7 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 	catch {close $in}
 	catch {close $filein}
 	if [info exists tfile] {
-	    catch {exec rm $tfile}
+	    File_Delete $tfile
 	}
 	focus $t
 	SeditDirty $t
@@ -209,39 +255,139 @@ proc SeditCiteSelection { draft t } {
 	SeditMsg $t "No selection"
 	return
     }
-    regsub -all "\n\n+" $line \x01 line
-    set space 0
-    foreach line [split $line \x01] {
-	regsub -all {]|[.^$*+|()\[\\]} $sedit(pref,replPrefix) {\\&} pattern
-	regsub -all "\n *($pattern)?" $line { } line
 
-	set limit 70
-	set cutoff 50
-
-	if {! $space} {
-	    $t insert insert "\n$address said:\n"
-	    set space 1
-	} else {
-	    $t insert insert \n
-	}
-	while {[string length $line] > $limit} {
-	    for {set c [expr $limit-1]} {$c >= $cutoff} {incr c -1} {
-		set char [string index $line $c]
-		if [regexp \[\ \t\n>/\] $char] {
-		    break
-		}
-	    }
-	    if {$c < $cutoff} {
-		set c [expr $limit-1]
-	    }
-	    set newline [string range $line 0 $c]
-	    $t insert insert "$sedit(pref,replPrefix)$newline\n"
-	    incr c
-	    set line [string range $line $c end]
-	}
-	$t insert insert "$sedit(pref,replPrefix)$line\n"
+    # check for 8bit characters in the selection
+    if [regexp "\[\x80-\xff\]" $line] {
+	set sedit($t,8bit) 1
     }
 
+    $t insert insert "\n$address said:\n"
+
+    # Divide selection into groups separated by blank lines
+    # Control-A is used as a pseudo-newline
+    regsub -all "\n\n+" $line \x01 line
+
+    set space ""
+    set limit 70
+    set cutoff 50
+
+    regsub -all {]|[.^$*+|()\[\\]} $sedit(pref,replPrefix) {\\&} pattern
+    foreach line [split $line \x01] {
+	# Preserve line breaks that start with white space or the replPrefix
+	regsub -all "\n(\[\ \t\n\]+)" $line \x01\\1 line
+	regsub -all "\n$pattern" $line \x01 line
+	# Eliminate leading replPrefix
+	regsub  "^$pattern" $line "" line
+	# Eliminate other line breaks
+	regsub -all \n $line " " line
+
+	foreach line [split $line \x01] {
+	    while {[string length $line] > $limit} {
+		set hit 0
+		for {set c $limit} {$c >= $cutoff} {incr c -1} {
+		    set char [string index $line $c]
+		    if [regexp \[\ \t\n>/\] $char] {
+			set hit 1
+			break
+		    }
+		}
+		if !$hit {
+		    set c $limit
+		}
+		set newline [string trimright [string range $line 0 $c]]
+		$t insert insert "$sedit(pref,replPrefix)$newline\n"
+		set space \n
+		incr c
+		set line [string range $line $c end]
+	    }
+	    $t insert insert "$sedit(pref,replPrefix)$line\n"
+	}
+	$t insert insert $space
+    }
+
+}
+
+proc Sedit_FormatParagraph { t } {
+    global sedit address
+    if [catch {$t index "sel.first linestart"} first] {
+	set first [$t index "insert linestart"]
+	set last [$t index "insert lineend"]
+	while 1 {
+	    set line [$t get $first "$first lineend"]
+	    set len [string length $line]
+	    if {$len == 0} {
+		break
+	    }
+	    set first [$t index "$first - 1line"]
+	    if {[regexp ^-- $line] || [$t compare $first <= hlimit]} {
+		break
+	    }
+	}
+	set first [$t index "$first + 1line"]
+	while 1 {
+	    set line [$t get "$last linestart" $last]
+	    set len [string length $line]
+	    if {($len == 0) || [regexp ^-- $line]} {
+		set last [$t index "$last - 1line lineend +1char"]
+		break
+	    }
+	    set nlast [$t index "$last + 1line lineend"]
+	    if {[$t compare $nlast == $last]} {
+		break
+	    }
+	    set last $nlast
+	}
+    } else {
+	set last [$t index "sel.last lineend"]
+    }
+    set line [$t get $first "$last -1char"]
+    set tags [$t tag names $first]
+    $t delete $first $last
+    $t mark set insert $first
+
+    # Divide selection into groups separated by blank lines
+    # Control-A is used as a pseudo-newline
+    regsub -all "\n\n+" $line \x01 line
+
+    set space ""
+    set limit $sedit(lineLength)
+    set cutoff 0
+
+    # Escape Tcl specials
+    regsub -all {]|[.^$*+|()\[\\]} $sedit(pref,replPrefix) {\\&} pattern
+    foreach line [split $line \x01] {
+	# Preserve line breaks that start with white space or the replPrefix
+	regsub -all "\n(\[\ \t\n\]+)" $line \x01\\1 line
+	regsub -all "\n$pattern" $line \x01 line
+	# Eliminate other line breaks
+	regsub -all " *\n" $line " " line
+
+	$t insert insert $space
+	set space \n
+
+	foreach line [split $line \x01] {
+	    while {[string length $line] > $limit} {
+		set hit 0
+		for {set c $limit} {$c >= $cutoff} {incr c -1} {
+		    set char [string index $line $c]
+		    if [regexp \[\ \t\n>/\] $char] {
+			set hit 1
+			break
+		    }
+		}
+		if !$hit {
+		    set c $limit
+		}
+		set newline [string trimright [string range $line 0 $c]]
+		$t insert insert "$newline\n" $tags
+		incr c
+		set line [string range $line $c end]
+	    }
+	    $t insert insert "$line\n" $tags
+	}
+    }
+    $t mark set insert "insert -1char"
+
 }
 
 proc SeditInsertFileDirect { draft t } {
@@ -292,7 +438,7 @@ proc SeditInsertExternalDialog { draft t } {
 					[exec uname -n]]
     close $fp
     eval {SeditInsertFile $draft $t $tmpfname} $options
-    catch {exec rm $tmpfname}
+    File_Delete $tmpfname
 }
 proc SeditExternalDialog { t name } {
     global sedit
@@ -306,7 +452,7 @@ proc SeditExternalDialog { t name } {
     set sedit($t,desc) [file tail $name]
     Widget_LabeledEntry $f.e0 Description: sedit($t,desc)
 
-    catch {exec hostname} sedit($t,extsite)
+    catch {exec uname -n} sedit($t,extsite)
     Widget_LabeledEntry $f.e1 Site: sedit($t,extsite)
 
     set sedit($t,extdirectory) [file dirname $name]
@@ -315,9 +461,9 @@ proc SeditExternalDialog { t name } {
     set sedit($t,extname) [file tail $name]
     Widget_LabeledEntry $f.e3 "File name" sedit($t,extname)
     Widget_BindEntryCmd $f.e3.entry <Return> \
-	[list SeditTweakContentType sedit($t,extname) sedit($t,exttype)]
+	[list SeditTweakContentType sedit($t,extname) sedit($t,exttype) $name]
 
-    SeditTweakContentType sedit($t,extname) sedit($t,exttype)
+    SeditTweakContentType sedit($t,extname) sedit($t,exttype) $name
     Widget_LabeledEntry $f.e4 "Content-Type:" sedit($t,exttype)
 
     set sedit($t,trans-mode) image
@@ -358,15 +504,15 @@ proc SeditExternalDialog { t name } {
 	set sedit($t,type) "message/external-body;\n\tname=\"$sedit($t,extname)\";\n\tsite=\"$sedit($t,extsite)\";\n\taccess-type=$sedit($t,extaccesstype);\n\tdirectory=\"$sedit($t,extdirectory)\";\n\tmode=\"$sedit($t,trans-mode)\""
     }
 
-    Exmh_Debug SeditExternalDialog $sedit($t,newpart) $sedit($t,encoding) $sedit($t,type) $sedit($t,desc)
     return [list $sedit($t,newpart) $sedit($t,encoding) $sedit($t,type) $sedit($t,desc)]
 }
-proc SeditTweakContentType { nameVar contentVar } {
+proc SeditTweakContentType { nameVar contentVar filenameOrig } {
+    global sedit
     upvar #0 $nameVar name
     upvar #0 $contentVar content
-    if [catch {SeditGuessContentType $name} content] {
+    if [catch {SeditGuessContentType $filenameOrig} content] {
 	Exmh_Status $content
-	set content text/plain
+	set content $sedit(defaultType)
     }
 }
 proc SeditFormatDialog { t name } {
@@ -375,7 +521,7 @@ proc SeditFormatDialog { t name } {
 
     if [catch {SeditGuessContentType $name} sedit($t,type)] {
 	Exmh_Status $sedit($t,type)
-	set sedit($t,type) "text/plain; name=\"[file tail $name]\""
+	set sedit($t,type) "$sedit(defaultType); name=\"[file tail $name]\""
     }
     message $f.msg1 -text "File Insert [file tail $name]" -aspect 1000
     pack $f.msg1 -side top -fill both
@@ -423,7 +569,6 @@ proc SeditFormatDialog { t name } {
     }
     Widget_PlaceDialog $t $f
     tkwait window $f
-    Exmh_Debug SeditFormatDialog $sedit($t,newpart) $sedit($t,encoding) $sedit($t,type) $sedit($t,desc)
     return [list $sedit($t,newpart) $sedit($t,encoding) $sedit($t,type) $sedit($t,desc)]
 }
 proc SeditFormatNewPart { t f {doit 0} } {
@@ -434,7 +579,8 @@ proc SeditFormatNewPart { t f {doit 0} } {
 proc SeditSpell { draft f t } {
     global tk_version sedit editor wish
     set parent [file root $f]
-    if {[catch {destroy $parent.spell}] == 0} {
+    if {[winfo exists $parent.spell]} {
+	destroy $parent.spell
 	return
     }
     # Do an unformatted save so spell gets the right info
@@ -443,7 +589,7 @@ proc SeditSpell { draft f t } {
 
     switch -- $sedit(spell) {
 	ispell {set prog {exmh-async xterm -e ispell}}
-	custom {set prog $sedit(spellCustom)}
+	custom {set prog $editor(spell)}
 	default {set prog spell}
     }
     if [string match exmh-async* $prog] {
@@ -457,7 +603,7 @@ proc SeditSpell { draft f t } {
 	puts $out "wm withdraw ."
 	puts $out "catch \{"
 	puts $out "exec [lrange $editor(spell) 1 end] $path"
-	puts $out "catch \}"
+	puts $out "\}"
 	puts $out [list send [winfo name .] [list SeditReplaceBody $t $path]]
 	puts $out "exec rm -f $path"
 	puts $out "exec rm -f $script"
@@ -519,7 +665,7 @@ proc Sedit_FindMatch { L string } {
     return [FindTextMatch $sedit(searchWidget) $L $string]
 }
 proc SeditGuessContentType { filenameOrig } {
-    global mimeType
+    global exmh mimeType sedit
 
     set filename [string tolower $filenameOrig]
     set type {}
@@ -527,7 +673,7 @@ proc SeditGuessContentType { filenameOrig } {
 	if ![info exists mimeType] {
 	    SeditLoadMimeTypes
 	}
-	if [regexp -- {^[1-9][0-9]*$} [file tail $filename]] {
+	if [regexp -- {^([1-9][0-9]*|@)$} [file tail $filename]] {
 	    return message/rfc822
 	}
 	set suffix [file extension $filename]
@@ -542,32 +688,22 @@ proc SeditGuessContentType { filenameOrig } {
 	}
     }
     if {[string length $type] == 0} {
-	if [catch {exec file $filenameOrig} result] {
-	    Exmh_Debug $result
-	    return application/octet-stream
-	} else {
-	    Exmh_Debug $result
-	    regsub ^$filenameOrig: $result {} result
-	    switch -regexp -- $result {
-		{English text}	{return text/plain}
-		{ascii text}	{return text/plain}
-		{script}	{return application/x-script}
-		{core|data}	{return application/octet-stream}
-		default		{return application/octet-stream}
-	    }
+	if {[string length [set type [Mime_Magic $filenameOrig]]] == 0} {
+	    return $sedit(defaultType)
 	}
     }
     return $type
 }
 proc SeditLoadMimeTypes {} {
-    global exmh mimeType env
+    global exmh mimeType env mimetypes_default
     # A few defaults
     set mimeType(.au)  audio/basic
     set mimeType(.gif) image/gif
     set mimeType(.ps)  application/postscript
     set mimeType(.txt) text/plain
-    SeditReadMimeTypes $exmh(library)/mime.types
-    SeditReadMimeTypes $exmh(library)/local.mime.types
+    SeditReadMimeTypes $exmh(library)/mime.types		;# depreciated
+    SeditReadMimeTypes $exmh(library)/local.mime.types		;# depreciated
+    SeditReadMimeTypes $mimetypes_default			;# new
     SeditReadMimeTypes $env(HOME)/.mime.types
     SeditReadMimeTypes $exmh(userLibrary)/user.mime.types
 }
@@ -593,10 +729,66 @@ proc SeditReadMimeTypes {file} {
     }
 }
 
+proc SeditCheckForIsigHeader { t hdrline } {
+    # Check whether there's an existing X-Exmh-Isig-Folder or
+    # X-Exmh-Isig-CompType header line
+    if {[catch {set end [$t index hlimit]}] &&
+	[catch {set end [$t index header]}]} {
+	    set end end
+    }
+    set X [$t get 1.0 $end]
+    if {![regexp -nocase "(^|\n)x-exmh-isig-$hdrline:\[ \t\]*(\[^\n\]*)\n" $X bin1 bin2 cont]} {
+	return {}
+    }
+    return $cont
+}
+proc SeditSetIsigHeader { t hook svar evar } {
+    global exmh sedit intelligentSign
+    set cont [eval SeditCheckForIsigHeader $t $hook]
+    if {$cont == {}} {
+	set sedit($t,$svar) $exmh($evar)
+	if {$intelligentSign(showhdrs)} {
+	    $t insert 1.0 "X-Exmh-Isig-$hook: $sedit($t,$svar)\n"
+	}
+    } else {
+	set sedit($t,$svar) $cont
+	if {!$intelligentSign(showhdrs)} {
+	    if {[catch {set end [$t index hlimit]}] &&
+		[catch {set end [$t index header]}]} {
+		    set end end
+	    }
+	    for {set spos [$t search -regexp -nocase "^x-exmh-isig-$hook:.*\$" 1.0 $end]} {$spos != {}} {} {
+		set sidx [$t index $spos]
+		regexp {([0-9]*)\..*} $sidx bin1 line
+		incr line
+		$t delete $spos $line.0
+		set spos [$t search -regexp -nocase "^x-exmh-isig-$hook:.*\$" 1.0 $end]
+	    }
+	}
+    }
+}
+proc SeditCheckForIsigHeaders { t } {
+    global sedit
+    set cont [SeditCheckForIsigHeader $t CompType]
+    if {$cont != {}} {
+	set sedit($t,isigc) $cont
+    }
+    set cont [SeditCheckForIsigHeader $t Folder]
+    if {$cont != {}} {
+	set sedit($t,isigf) $cont
+    }
+}
+proc SeditSetIsigHeaders { t } {
+    global exmh
+    if {![info exists exmh(ctype)]} {
+	set exmh(ctype) {unknown}
+    }
+    SeditSetIsigHeader $t CompType isigc ctype
+    set exmh(ctype) {unknown}
+    SeditSetIsigHeader $t Folder isigf folder
+}
+
 # proc SeditClip: Use Clip from Sedit as a previewer
-#
-# Requirements: Need extended Msg_clip procdure (see msg.tcl.patch)
-#
 
 proc SeditClip {draft t} {
     global mhProfile
@@ -626,11 +818,18 @@ proc Sedit_Mailto { url } {
 }
 # Run MHN now to format a message
 proc SeditMHN {draft t} {
-    global env sedit
+    global env sedit editor
 
     set format $sedit($t,format)
-    set sedit($t,format) 0
-    if [catch {SeditSave $draft $t} err] {
+    set sedit($t,format) Never
+# add these two clauses from SeditSend
+    if {$sedit($t,mhn)} {
+      SeditFixupMhn $draft $t
+    }
+    if {$sedit(iso)} {
+      SeditFixupCharset $draft $t
+    }
+    if [catch {SeditSave $draft $t {} 0} err] {
 	SeditMsg $t $err
 	set sedit($t,format) $format
 	return
@@ -638,7 +837,7 @@ proc SeditMHN {draft t} {
     set sedit($t,format) $format
 
     set env(mhdraft) $draft
-    if [catch {exec mhn $draft} err] {
+    if [catch {exec $editor(mhn) $draft} err] {
 	SeditMsg $t $err
     } else {
 	if [catch {open $draft r} in] {
@@ -653,4 +852,47 @@ proc SeditMHN {draft t} {
 	}
     }
 }
+proc SeditExternalCmd { draft t cmd } {
+    # Save message, process with external command, and reload
+    # last argument to command will be draft file name.
+    SeditSave $draft $t
+    if [catch {eval exec $cmd $draft} err] {
+	Exmh_Debug "$err while executing external command."
+    } else {
+	if [catch {open $draft r} in] {
+	    SeditMsg $t "Cannot open $draft"
+	} else {
+	    $t delete 1.0 end
+	    SeditMimeReset $t
+	    $t insert 1.0 [read $in]
+	    close $in
+	    SeditPositionCursor $t
+	    SeditMimeParse $t
+	}
+    }
+}
 
+proc SeditAttachQuotedMessage { draft t name } {
+	global sedit
+	if {$name != ""} {
+		if [file readable $name] {
+			set options [SeditFormatDialog $t $name]
+			eval {SeditInsertFile $draft $t $name} $options
+		} else {
+			SeditMsg $t "Cannot read $name"
+		}
+	}
+}
+
+proc SeditInsertMessageDialog { draft t } {
+	global sedit msg
+	set name [FSBox "Select message" $msg(path)]
+	if {$name != ""} {
+		if [file readable $name] {
+		set options [SeditFormatDialog $t $name]
+		eval {SeditInsertFile $draft $t $name} $options
+		} else {
+			SeditMsg $t "Cannot read $name"
+		}
+	}
+}
